@@ -12,6 +12,7 @@ const FORCE_SRAM_CHECKSUMS: &[u32] = &[
     0x8135702C, // NHL 96 (UE)
     0xF509145F, // Might and Magic: Gates to Another World (UE)
     0x6EF7104A, // Might and Magic III: Isles of Terra (U) (Proto)
+    0x2491DF2F, // NBA Action '94 (USA) (Beta) (1994-01-04)
 ];
 
 const SONIC_AND_KNUCKLES_SERIAL: &[u8] = b"GM MK-1563 ";
@@ -56,8 +57,8 @@ impl Ram {
 
         let ram_header_bytes = &rom[0x1B0..0x1BC];
 
-        // RAM header should always start with ASCII "RA" followed by $F8 and $20
-        if ram_header_bytes[..4] != [b'R', b'A', 0xF8, 0x20] {
+        // RAM header should always start with ASCII "RA" followed by RAM type byte and $20
+        if !matches!(&ram_header_bytes[..4], &[b'R', b'A', _, 0x20]) {
             return None;
         }
 
