@@ -84,13 +84,6 @@ impl HorizontalDisplaySize {
         }
     }
 
-    pub const fn mclk_cycles_per_pixel(self) -> u64 {
-        match self {
-            Self::ThirtyTwoCell => 10,
-            Self::FortyCell => 8,
-        }
-    }
-
     // Length in sprites
     pub const fn sprite_table_len(self) -> u16 {
         match self {
@@ -227,10 +220,6 @@ impl InterlacingMode {
             Self::InterlacedDouble => 4,
         }
     }
-
-    pub const fn is_interlaced(self) -> bool {
-        matches!(self, Self::Interlaced | Self::InterlacedDouble)
-    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode)]
@@ -602,7 +591,7 @@ impl Registers {
                 // Register #15: VRAM address auto increment
                 self.data_port_auto_increment = value.into();
 
-                log::trace!("  Data port auto increment: {:02X}", value);
+                log::trace!("  Data port auto increment: {value:02X}");
             }
             16 => {
                 // Register #16: Scroll size
@@ -694,13 +683,13 @@ impl Registers {
         self.window_horizontal_mode.h_range(self.window_x_position, active_display_pixels)
     }
 
-    pub fn dma_length(&self) -> u32 {
-        if self.dma_length > 0 {
-            self.dma_length.into()
-        } else {
-            // DMA length of 0 is treated as 65536
-            65536
-        }
+    pub fn masked_window_nametable_addr(&self) -> u16 {
+        // A11 is ignored in H40 mode; Cheese Cat-Astrophe depends on this
+        let mask = match self.horizontal_display_size {
+            HorizontalDisplaySize::ThirtyTwoCell => 0xF800,
+            HorizontalDisplaySize::FortyCell => 0xF000,
+        };
+        self.window_base_nt_addr & mask
     }
 
     pub fn masked_sprite_attribute_table_addr(&self) -> u16 {
