@@ -54,8 +54,8 @@ impl Encode for Cartridge {
     }
 }
 
-impl Decode for Cartridge {
-    fn decode<D: Decoder>(decoder: &mut D) -> Result<Self, DecodeError> {
+impl<Context> Decode<Context> for Cartridge {
+    fn decode<D: Decoder<Context = Context>>(decoder: &mut D) -> Result<Self, DecodeError> {
         let timing_mode = Decode::decode(decoder)?;
         let prg_ram = Decode::decode(decoder)?;
         let has_ram_battery = Decode::decode(decoder)?;
@@ -74,8 +74,10 @@ impl Decode for Cartridge {
     }
 }
 
-impl<'de> BorrowDecode<'de> for Cartridge {
-    fn borrow_decode<D: BorrowDecoder<'de>>(decoder: &mut D) -> Result<Self, DecodeError> {
+impl<'de, Context> BorrowDecode<'de, Context> for Cartridge {
+    fn borrow_decode<D: BorrowDecoder<'de, Context = Context>>(
+        decoder: &mut D,
+    ) -> Result<Self, DecodeError> {
         let timing_mode = BorrowDecode::borrow_decode(decoder)?;
         let prg_ram = BorrowDecode::borrow_decode(decoder)?;
         let has_ram_battery = BorrowDecode::borrow_decode(decoder)?;
@@ -465,20 +467,14 @@ impl INesHeader {
 
         // All iNES headers should begin with this 4-byte sequence, which is "NES" followed by the
         // character that MS-DOS used for EOF
-        if header[..4] != [0x4E, 0x45, 0x53, 0x1A] {
+        if header[..4] != [b'N', b'E', b'S', 0x1A] {
             return Err(CartridgeFileError::Format);
         }
 
         let format =
             if header[7] & 0x0C == 0x08 { FileFormat::Nes2Point0 } else { FileFormat::INes };
-
-        let prg_rom_size = {
-            let mut prg_rom_size_16kb = u32::from(header[4]);
-            if format == FileFormat::Nes2Point0 {
-                prg_rom_size_16kb |= u32::from(header[9] & 0x0F) << 8;
-            }
-            16 * 1024 * prg_rom_size_16kb
-        };
+        let has_trainer = header[6].bit(2);
+        let mapper_number = u16::from((header[7] & 0xF0) | ((header[6] & 0xF0) >> 4));
 
         let chr_rom_size = {
             let mut chr_rom_size_8kb = u32::from(header[5]);
@@ -488,6 +484,23 @@ impl INesHeader {
             8 * 1024 * chr_rom_size_8kb
         };
 
+        let prg_rom_size = {
+            let mut prg_rom_size_16kb = u32::from(header[4]);
+            if format == FileFormat::Nes2Point0 {
+                prg_rom_size_16kb |= u32::from(header[9] & 0x0F) << 8;
+            }
+            let prg_rom_size = 16 * 1024 * prg_rom_size_16kb;
+
+            // Hack: Galaxian (J) only has 8KB of PRG ROM, and the iNES header usually contains a
+            // nonsensical PRG ROM size because it can't represent this
+            if should_apply_8kb_prg_rom_hack(file_bytes, mapper_number, has_trainer, chr_rom_size) {
+                log::info!("Ignoring PRG ROM size in header of {prg_rom_size} bytes");
+                8 * 1024
+            } else {
+                prg_rom_size
+            }
+        };
+
         if header.len() + (prg_rom_size + chr_rom_size) as usize > file_bytes.len() {
             return Err(CartridgeFileError::InvalidRomSize {
                 file_size: file_bytes.len() as u32,
@@ -496,10 +509,6 @@ impl INesHeader {
             });
         }
 
-        let has_trainer = header[6].bit(2);
-
-        let mapper_number = u16::from((header[7] & 0xF0) | ((header[6] & 0xF0) >> 4));
-
         let chr_type = if chr_rom_size == 0 { ChrType::RAM } else { ChrType::ROM };
 
         let nametable_mirroring = if header[6].bit(0) {
@@ -534,7 +543,7 @@ impl INesHeader {
                 }
             }
             FileFormat::INes => {
-                if header[0].bit(0) {
+                if header[9].bit(0) {
                     TimingMode::Pal
                 } else {
                     TimingMode::Ntsc
@@ -576,6 +585,24 @@ impl INesHeader {
     }
 }
 
+// Check whether to assume the cartridge has 8KB of PRG ROM, which the iNES format cannot represent
+fn should_apply_8kb_prg_rom_hack(
+    file_bytes: &[u8],
+    mapper_number: u16,
+    has_trainer: bool,
+    chr_rom_size: u32,
+) -> bool {
+    if mapper_number != 0 {
+        // Only apply this hack for mapper 0
+        return false;
+    }
+
+    let trainer_len = if has_trainer { 512 } else { 0 };
+    let header_len = 16 + trainer_len;
+
+    file_bytes.len().saturating_sub(header_len).saturating_sub(chr_rom_size as usize) == 8 * 1024
+}
+
 fn determine_prg_ram_size(header: &[u8], mapper_number: u16, format: FileFormat) -> u32 {
     let prg_ram_size = match format {
         FileFormat::Nes2Point0 => {
@@ -642,14 +669,9 @@ pub(crate) fn from_ines_file(
         }
     }
 
-    let prg_ram = if let Some(sav_bytes) = &sav_bytes {
-        if sav_bytes.len() == header.prg_ram_size as usize {
-            sav_bytes.clone()
-        } else {
-            vec![0; header.prg_ram_size as usize]
-        }
-    } else {
-        vec![0; header.prg_ram_size as usize]
+    let prg_ram = match &sav_bytes {
+        Some(sav_bytes) if sav_bytes.len() == header.prg_ram_size as usize => sav_bytes.clone(),
+        _ => vec![0xFF; header.prg_ram_size as usize],
     };
 
     let timing_mode = forced_timing_mode.unwrap_or(header.timing_mode);
