@@ -298,6 +298,7 @@ type SpriteLineBuffer = [SpriteData; SCREEN_WIDTH as usize];
 #[derive(Debug, Clone, Encode, Decode)]
 pub struct PpuState {
     timing_mode: TimingMode,
+    pub ntsc_crop_vertical_overscan: bool,
     frame_buffer: Box<FrameBuffer>,
     registers: InternalRegisters,
     bg_buffers: BgBuffers,
@@ -311,9 +312,10 @@ pub struct PpuState {
 }
 
 impl PpuState {
-    pub fn new(timing_mode: TimingMode) -> Self {
+    pub fn new(timing_mode: TimingMode, ntsc_crop_vertical_overscan: bool) -> Self {
         Self {
             timing_mode,
+            ntsc_crop_vertical_overscan,
             frame_buffer: vec![
                 [(0, ColorEmphasis::default()); SCREEN_WIDTH as usize];
                 MAX_SCREEN_HEIGHT as usize
@@ -359,7 +361,10 @@ impl PpuState {
         bus: &mut PpuBus<'_>,
     ) {
         self.frame_buffer[y as usize][x as usize] = (pixel, color_emphasis);
-        bus.handle_pixel_rendered(pixel, x, y, self.timing_mode);
+
+        let display_mode =
+            if !self.ntsc_crop_vertical_overscan { TimingMode::Pal } else { self.timing_mode };
+        bus.handle_pixel_rendered(pixel, x, y, display_mode);
     }
 }
 
@@ -459,7 +464,7 @@ fn get_color_mask(registers: &PpuRegisters) -> u8 {
 /// memory-mapped PPU regsiters.
 pub fn reset(state: &mut PpuState, bus: &mut PpuBus<'_>) {
     let vram_address = state.registers.vram_address;
-    *state = PpuState::new(state.timing_mode);
+    *state = PpuState::new(state.timing_mode, state.ntsc_crop_vertical_overscan);
     state.registers.vram_address = vram_address;
 
     bus.reset();
@@ -470,7 +475,7 @@ fn process_scanline(state: &mut PpuState, bus: &mut PpuBus<'_>, remove_sprite_li
     let dot = state.dot;
     let timing_mode = state.timing_mode;
 
-    log::trace!("Rendering at scanline {} dot {}", scanline, dot);
+    log::trace!("Rendering at scanline {scanline} dot {dot}");
 
     if state.pending_sprite_0_hit {
         // If sprite 0 hit triggered on the last cycle, set the flag in PPUSTATUS
@@ -554,6 +559,12 @@ fn process_scanline(state: &mut PpuState, bus: &mut PpuBus<'_>, remove_sprite_li
 
                     fetch_sprite_tile_data(bus, scanline, dot, &mut state.sprite_buffers);
 
+                    // OAMADDR is repeatedly reset to 0 during these dots on rendering scanlines.
+                    // Ghostbusters II depends on this due to sometimes running OAM DMAs that don't
+                    // finish before the pre-render scanline begins, and it never explicitly writes
+                    // to OAMADDR after boot
+                    bus.get_ppu_registers_mut().set_oam_addr(0);
+
                     if dot == 320 {
                         if remove_sprite_limit {
                             finish_sprite_tile_fetches(
