@@ -8,26 +8,28 @@ use crate::rf5c164::Rf5c164;
 use bincode::{Decode, Encode};
 use cdrom::CdRomError;
 use cdrom::reader::{CdRom, CdRomFileFormat};
-use genesis_core::input::{GenesisButton, InputState};
+use genesis_config::{GenesisButton, GenesisRegion, PcmInterpolation};
+use genesis_core::input::InputState;
 use genesis_core::memory::{MainBus, MainBusSignals, MainBusWrites, Memory};
 use genesis_core::timing::CycleCounters;
 use genesis_core::vdp::{Vdp, VdpTickEffect};
-use genesis_core::ym2612::{Ym2612, YmTickEffect};
-use genesis_core::{GenesisEmulatorConfig, GenesisInputs, GenesisRegion};
+use genesis_core::ym2612::Ym2612;
+use genesis_core::{GenesisEmulatorConfig, GenesisInputs};
 use jgenesis_common::frontend::{
     AudioOutput, Color, EmulatorConfigTrait, EmulatorTrait, PartialClone, Renderer, SaveWriter,
     TickEffect, TimingMode,
 };
-use jgenesis_proc_macros::{ConfigDisplay, EnumAll, EnumDisplay, EnumFromStr};
+use jgenesis_proc_macros::ConfigDisplay;
 use m68000_emu::M68000;
-use smsgg_core::psg::{Sn76489, Sn76489TickEffect, Sn76489Version};
+use smsgg_config::Sn76489Version;
+use smsgg_core::psg::{Sn76489, Sn76489TickEffect};
 use std::fmt::{Debug, Display};
 use std::num::{NonZeroU16, NonZeroU64};
 use std::path::Path;
 use thiserror::Error;
 use z80_emu::Z80;
 
-pub const DEFAULT_SUB_CPU_DIVIDER: u64 = 4;
+pub const DEFAULT_SUB_CPU_DIVIDER: u64 = genesis_config::DEFAULT_SUB_CPU_DIVIDER;
 
 const NTSC_GENESIS_MASTER_CLOCK_RATE: u64 = 53_693_175;
 const PAL_GENESIS_MASTER_CLOCK_RATE: u64 = 53_203_424;
@@ -68,30 +70,6 @@ pub enum SegaCdError<RErr, AErr, SErr> {
 
 pub type SegaCdResult<T, RErr, AErr, SErr> = Result<T, SegaCdError<RErr, AErr, SErr>>;
 
-#[derive(
-    Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode, EnumDisplay, EnumFromStr, EnumAll,
-)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-#[cfg_attr(feature = "clap", derive(jgenesis_proc_macros::CustomValueEnum))]
-pub enum PcmInterpolation {
-    #[default]
-    None,
-    Linear,
-    CubicHermite,
-    CubicHermite6Point,
-}
-
-#[derive(
-    Debug, Clone, Copy, PartialEq, Eq, Default, Encode, Decode, EnumDisplay, EnumFromStr, EnumAll,
-)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-#[cfg_attr(feature = "clap", derive(jgenesis_proc_macros::CustomValueEnum))]
-pub enum PcmLowPassFilter {
-    None,
-    #[default]
-    SegaCd,
-}
-
 #[derive(Debug, Clone, Copy, Encode, Decode, ConfigDisplay)]
 pub struct SegaCdEmulatorConfig {
     #[cfg_display(skip)]
@@ -101,7 +79,8 @@ pub struct SegaCdEmulatorConfig {
     pub load_disc_into_ram: bool,
     pub disc_drive_speed: NonZeroU16,
     pub sub_cpu_divider: NonZeroU64,
-    pub pcm_low_pass: PcmLowPassFilter,
+    pub pcm_lpf_enabled: bool,
+    pub pcm_lpf_cutoff: u32,
     pub apply_genesis_lpf_to_pcm: bool,
     pub apply_genesis_lpf_to_cd_da: bool,
     pub pcm_enabled: bool,
@@ -154,6 +133,8 @@ macro_rules! new_main_bus {
             &mut $self.psg,
             &mut $self.ym2612,
             &mut $self.input,
+            &mut $self.cycles,
+            $self.main_cpu.next_opcode(),
             $self.timing_mode,
             MainBusSignals { m68k_reset: $m68k_reset },
             std::mem::take(&mut $self.main_bus_writes),
@@ -242,7 +223,7 @@ impl SegaCdEmulator {
         let z80 = Z80::new();
         let vdp = Vdp::new(timing_mode, emulator_config.genesis.to_vdp_config());
         let graphics_coprocessor = GraphicsCoprocessor::new();
-        let ym2612 = Ym2612::new(emulator_config.genesis);
+        let ym2612 = Ym2612::new_from_config(&emulator_config.genesis);
         let psg = Sn76489::new(Sn76489Version::Standard);
         let pcm = Rf5c164::new(&emulator_config);
         let input = InputState::new(
@@ -308,6 +289,7 @@ impl SegaCdEmulator {
 
     fn render_frame<R: Renderer>(&self, renderer: &mut R) -> Result<(), R::Err> {
         genesis_core::render_frame(
+            self.timing_mode,
             &self.vdp,
             self.config.genesis.aspect_ratio,
             self.config.genesis.adjust_aspect_ratio_in_2x_resolution,
@@ -315,11 +297,18 @@ impl SegaCdEmulator {
         )
     }
 
+    #[inline]
     #[must_use]
     pub fn disc_title(&self) -> &str {
         &self.disc_title
     }
 
+    #[inline]
+    #[must_use]
+    pub fn timing_mode(&self) -> TimingMode {
+        self.timing_mode
+    }
+
     pub fn remove_disc(&mut self) {
         self.memory.medium_mut().remove_disc();
         self.disc_title = "(no disc)".into();
@@ -382,28 +371,32 @@ impl EmulatorTrait for SegaCdEmulator {
         let mut main_bus = new_main_bus!(self, m68k_reset: false);
 
         // Main 68000
-        let main_cpu_cycles = if self.cycles.m68k_wait_cpu_cycles != 0 {
-            self.cycles.take_m68k_wait_cpu_cycles()
+        let m68k_wait = main_bus.cycles.m68k_wait_cpu_cycles != 0;
+        let main_cpu_cycles = if m68k_wait {
+            main_bus.cycles.take_m68k_wait_cpu_cycles()
         } else {
             self.main_cpu.execute_instruction(&mut main_bus)
         };
-        let genesis_mclk_elapsed = self.cycles.record_68k_instruction(
+        let genesis_mclk_elapsed = main_bus.cycles.record_68k_instruction(
             main_cpu_cycles,
-            self.main_cpu.last_instruction_was_mul_or_div(),
+            m68k_wait,
+            main_bus.vdp.should_halt_cpu(),
         );
 
         // Z80
-        while self.cycles.should_tick_z80() {
-            self.z80.tick(&mut main_bus);
-            self.cycles.decrement_z80();
-        }
-
-        if main_bus.z80_accessed_68k_bus() {
-            self.cycles.record_z80_68k_bus_access();
+        while main_bus.cycles.should_tick_z80() {
+            if !main_bus.cycles.z80_halt {
+                self.z80.tick(&mut main_bus);
+            }
+            main_bus.cycles.decrement_z80();
         }
 
         self.main_bus_writes = main_bus.take_writes();
 
+        if !m68k_wait {
+            self.vdp.clear_interrupt_delays();
+        }
+
         self.sega_cd_mclk_cycle_product += genesis_mclk_elapsed * SEGA_CD_MASTER_CLOCK_RATE;
         let scd_mclk_elapsed = match self.timing_mode {
             TimingMode::Ntsc => {
@@ -475,12 +468,10 @@ impl EmulatorTrait for SegaCdEmulator {
         }
 
         // YM2612
-        while self.cycles.should_tick_ym2612() {
-            if self.ym2612.tick() == YmTickEffect::OutputSample {
-                let (ym2612_sample_l, ym2612_sample_r) = self.ym2612.sample();
-                self.audio_resampler.collect_ym2612_sample(ym2612_sample_l, ym2612_sample_r);
-            }
-            self.cycles.decrement_ym2612();
+        if self.cycles.has_ym2612_ticks() {
+            let ym2612_ticks = self.cycles.take_ym2612_ticks();
+            self.ym2612
+                .tick(ym2612_ticks, |(l, r)| self.audio_resampler.collect_ym2612_sample(l, r));
         }
 
         // RF5C164
@@ -530,7 +521,7 @@ impl EmulatorTrait for SegaCdEmulator {
         self.ym2612.reload_config(config.genesis);
         self.pcm.reload_config(config);
         self.input.reload_config(config.genesis);
-        self.audio_resampler.reload_config(*config);
+        self.audio_resampler.reload_config(self.timing_mode, *config);
         self.cycles.update_m68k_divider(config.genesis.clamped_m68k_divider());
         self.sub_cpu_divider = config.sub_cpu_divider.get();
 
@@ -549,7 +540,7 @@ impl EmulatorTrait for SegaCdEmulator {
         self.main_cpu.execute_instruction(&mut new_main_bus!(self, m68k_reset: true));
         self.memory.reset_z80_signals();
 
-        self.ym2612.reset(self.config.genesis);
+        self.ym2612.reset();
         self.pcm.disable();
 
         self.memory.medium_mut().reset();
@@ -565,7 +556,7 @@ impl EmulatorTrait for SegaCdEmulator {
     }
 
     fn target_fps(&self) -> f64 {
-        genesis_core::target_framerate(self.timing_mode)
+        genesis_core::target_framerate(&self.vdp, self.timing_mode)
     }
 
     fn update_audio_output_frequency(&mut self, output_frequency: u64) {
