@@ -5,6 +5,7 @@ pub mod cdd;
 
 use crate::api::{SegaCdEmulatorConfig, SegaCdLoadResult};
 use crate::cddrive::cdc::RchipDmaArgs;
+use crate::cddrive::cdd::CdModel;
 use crate::memory::wordram::WordRam;
 use crate::rf5c164::Rf5c164;
 use crate::{api, memory};
@@ -12,7 +13,7 @@ use bincode::{Decode, Encode};
 use cdc::Rchip;
 use cdd::CdDrive;
 use cdrom::reader::CdRom;
-use genesis_core::GenesisRegion;
+use genesis_config::GenesisRegion;
 use jgenesis_proc_macros::PartialClone;
 use std::array;
 
@@ -53,16 +54,16 @@ pub struct CdController {
     #[partial_clone(partial)]
     drive: CdDrive,
     rchip: Rchip,
-    sector_buffer: [u8; cdrom::BYTES_PER_SECTOR as usize],
+    sector_buffer: Box<[u8; cdrom::BYTES_PER_SECTOR as usize]>,
     prescaler: CdPrescaler,
 }
 
 impl CdController {
-    pub fn new(disc: Option<CdRom>, config: &SegaCdEmulatorConfig) -> Self {
+    pub fn new(disc: Option<CdRom>, model: CdModel, config: &SegaCdEmulatorConfig) -> Self {
         Self {
-            drive: CdDrive::new(disc, config),
+            drive: CdDrive::new(disc, model, config),
             rchip: Rchip::new(),
-            sector_buffer: array::from_fn(|_| 0),
+            sector_buffer: Box::new(array::from_fn(|_| 0)),
             prescaler: CdPrescaler::new(),
         }
     }
