@@ -1,29 +1,9 @@
 //! Code for handling Sega Master System / Game Gear controller input I/O registers
 
-use crate::api::SmsRegion;
+use crate::vdp::Vdp;
 use bincode::{Decode, Encode};
-use jgenesis_common::define_controller_inputs;
 use jgenesis_common::num::GetBit;
-
-define_controller_inputs! {
-    buttons: SmsGgButton {
-        Up -> up,
-        Left -> left,
-        Right -> right,
-        Down -> down,
-        Button1 -> button1,
-        Button2 -> button2,
-    },
-    non_gamepad_buttons: [Pause],
-    joypad: SmsGgJoypadState,
-    inputs: SmsGgInputs {
-        players: {
-            p1: Player::One,
-            p2: Player::Two,
-        },
-        buttons: [Pause -> pause],
-    },
-}
+use smsgg_config::{SmsGgInputs, SmsGgRegion};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Encode, Decode)]
 enum PinDirection {
@@ -47,12 +27,12 @@ pub struct InputState {
     port_a_th: PinDirection,
     port_b_tr: PinDirection,
     port_b_th: PinDirection,
-    region: SmsRegion,
+    region: SmsGgRegion,
     reset: bool,
 }
 
 impl InputState {
-    pub fn new(region: SmsRegion) -> Self {
+    pub fn new(region: SmsGgRegion) -> Self {
         Self {
             inputs: SmsGgInputs::default(),
             port_a_tr: PinDirection::Input,
@@ -72,11 +52,11 @@ impl InputState {
         self.inputs = inputs;
     }
 
-    pub fn region(&self) -> SmsRegion {
+    pub fn region(&self) -> SmsGgRegion {
         self.region
     }
 
-    pub fn set_region(&mut self, region: SmsRegion) {
+    pub fn set_region(&mut self, region: SmsGgRegion) {
         self.region = region;
     }
 
@@ -84,7 +64,12 @@ impl InputState {
         self.reset = reset;
     }
 
-    pub fn write_control(&mut self, value: u8) {
+    pub fn write_control(&mut self, value: u8, vdp: &mut Vdp) {
+        log::debug!("I/O control write {value:02X}");
+
+        let prev_a_th = self.port_a_th != PinDirection::Output(false);
+        let prev_b_th = self.port_b_th != PinDirection::Output(false);
+
         self.port_b_th =
             if value.bit(3) { PinDirection::Input } else { PinDirection::Output(value.bit(7)) };
         self.port_b_tr =
@@ -93,6 +78,12 @@ impl InputState {
             if value.bit(1) { PinDirection::Input } else { PinDirection::Output(value.bit(5)) };
         self.port_a_tr =
             if value.bit(0) { PinDirection::Input } else { PinDirection::Output(value.bit(4)) };
+
+        if (!prev_a_th && self.port_a_th != PinDirection::Output(false))
+            || (!prev_b_th && self.port_b_th != PinDirection::Output(false))
+        {
+            vdp.latch_h_counter_on_th_change();
+        }
     }
 
     pub fn port_dc(&self) -> u8 {
@@ -110,9 +101,9 @@ impl InputState {
 
     pub fn port_dd(&self) -> u8 {
         let port_b_th_bit =
-            u8::from(self.region == SmsRegion::International && self.port_b_th.bit(true)) << 7;
+            u8::from(self.region == SmsGgRegion::International && self.port_b_th.bit(true)) << 7;
         let port_a_th_bit =
-            u8::from(self.region == SmsRegion::International && self.port_a_th.bit(true)) << 6;
+            u8::from(self.region == SmsGgRegion::International && self.port_a_th.bit(true)) << 6;
         let port_b_tr_bit = u8::from(self.port_b_tr.bit(!self.inputs.p2.button2)) << 3;
 
         port_b_th_bit
