@@ -6,7 +6,7 @@ pub fn fake_encode(input: TokenStream) -> TokenStream {
     let input: DeriveInput = syn::parse(input).expect("Unable to parse input");
 
     let type_ident = &input.ident;
-    let gen = quote! {
+    let expanded = quote! {
         impl ::bincode::Encode for #type_ident {
             fn encode<E: ::bincode::enc::Encoder>(
                 &self,
@@ -17,24 +17,24 @@ pub fn fake_encode(input: TokenStream) -> TokenStream {
         }
     };
 
-    gen.into()
+    expanded.into()
 }
 
 pub fn fake_decode(input: TokenStream) -> TokenStream {
     let input: DeriveInput = syn::parse(input).expect("Unable to parse input");
 
     let type_ident = &input.ident;
-    let gen = quote! {
-        impl ::bincode::Decode for #type_ident {
-            fn decode<D: ::bincode::de::Decoder>(
+    let expanded = quote! {
+        impl<Context> ::bincode::Decode<Context> for #type_ident {
+            fn decode<D: ::bincode::de::Decoder<Context = Context>>(
                 _decoder: &mut D
             ) -> ::std::result::Result<Self, ::bincode::error::DecodeError> {
                 ::std::result::Result::Ok(Self::default())
             }
         }
 
-        impl<'de> ::bincode::BorrowDecode<'de> for #type_ident {
-            fn borrow_decode<D: ::bincode::de::BorrowDecoder<'de>>(
+        impl<'de, Context> ::bincode::BorrowDecode<'de, Context> for #type_ident {
+            fn borrow_decode<D: ::bincode::de::BorrowDecoder<'de, Context = Context>>(
                 _decoder: &mut D
             ) -> ::std::result::Result<Self, ::bincode::error::DecodeError> {
                 ::std::result::Result::Ok(Self::default())
@@ -42,5 +42,5 @@ pub fn fake_decode(input: TokenStream) -> TokenStream {
         }
     };
 
-    gen.into()
+    expanded.into()
 }
