@@ -1,7 +1,6 @@
 mod config;
 mod encode;
 mod enums;
-mod inputs;
 mod partialclone;
 
 use proc_macro::TokenStream;
@@ -96,7 +95,7 @@ pub fn custom_value_enum(input: TokenStream) -> TokenStream {
 /// assert_eq!(s, "config: \n  foo: 5\n  bar: asdf".to_owned());
 /// ```
 ///
-/// The `#[debug_fmt]` attribute can be used to indicate that a field should be formatted using its
+/// The `#[cfg_display(debug_fmt)]` attribute can be used to indicate that a field should be formatted using its
 /// `std::fmt::Debug` implementation rather than its `std::fmt::Display` implementation. For example:
 /// ```
 /// use jgenesis_proc_macros::ConfigDisplay;
@@ -107,7 +106,7 @@ pub fn custom_value_enum(input: TokenStream) -> TokenStream {
 /// #[derive(ConfigDisplay)]
 /// struct Config {
 ///     bar: bool,
-///     #[debug_fmt]
+///     #[cfg_display(debug_fmt)]
 ///     baz: NotDisplay,
 /// }
 ///
@@ -124,7 +123,7 @@ pub fn custom_value_enum(input: TokenStream) -> TokenStream {
 /// assert_eq!(s, "config: \n  bar: true\n  baz: NotDisplay(\"fdsa\")");
 /// ```
 ///
-/// The `#[indent_nested]` attribute allows indenting when printing a field that also implements
+/// The `#[cfg_display(indent_nested)]` attribute allows indenting when printing a field that also implements
 /// the `Display` trait through this macro:
 /// ```
 /// use jgenesis_proc_macros::ConfigDisplay;
@@ -137,7 +136,7 @@ pub fn custom_value_enum(input: TokenStream) -> TokenStream {
 /// #[derive(ConfigDisplay)]
 /// struct Outer {
 ///     bar: u32,
-///     #[indent_nested]
+///     #[cfg_display(indent_nested)]
 ///     inner: Inner,
 ///     baz: u32,
 /// }
@@ -181,7 +180,7 @@ pub fn custom_value_enum(input: TokenStream) -> TokenStream {
 ///
 /// This macro only supports structs with named fields and it will panic if applied to any other
 /// data type, including structs with no fields.
-#[proc_macro_derive(ConfigDisplay, attributes(debug_fmt, indent_nested))]
+#[proc_macro_derive(ConfigDisplay, attributes(cfg_display))]
 pub fn config_display(input: TokenStream) -> TokenStream {
     config::config_display(input)
 }
@@ -315,77 +314,3 @@ pub fn partial_clone(input: TokenStream) -> TokenStream {
 pub fn match_each_variant_macro(input: TokenStream) -> TokenStream {
     enums::match_each_variant_macro(input)
 }
-
-/// Define a button enum, joypad state struct, and console inputs struct.
-///
-/// As an example, this will define an enum `NesButton`, a struct `NesJoypadState`, and a struct
-/// `NesInputs`:
-///
-/// ```
-/// use jgenesis_common::input::Player;
-/// use jgenesis_proc_macros::define_controller_inputs;
-///
-/// define_controller_inputs! {
-///     enum NesButton {
-///         Up,
-///         Left,
-///         Right,
-///         Down,
-///         A,
-///         B,
-///         Start,
-///         Select,
-///     }
-///
-///     struct NesJoypadState {
-///         buttons!
-///     }
-///
-///     struct NesInputs {
-///         p1: Player::One,
-///         p2: Player::Two,
-///     }
-/// }
-///
-/// let mut inputs = NesInputs {
-///     p1: NesJoypadState {
-///         up: true,
-///         left: false,
-///         right: false,
-///         down: false,
-///         a: false,
-///         b: true,
-///         start: false,
-///         select: false,
-///     },
-///     p2: NesJoypadState::default(),
-/// };
-/// inputs = inputs.with_button(NesButton::A, Player::Two, true);
-/// assert!(inputs.p2.a);
-/// ```
-///
-/// The `NesButton` enum will have one variant per button in `buttons`.
-///
-/// The `NesJoypadState` struct will have a `bool` field for each button, with the field name equal
-/// to the lowercased variant name. It will also have the following two methods defined for setting
-/// fields from button enum values:
-/// * `set_button(&mut self, button: NesButton, pressed: bool)`
-/// * `with_button(self, button: NesButton, pressed: bool) -> Self`
-///
-/// The `NesInputs` struct will have two fields: `p1` and `p2`, both of type `NesJoypadState`. It
-/// will also have the following two methods defined for setting fields from button/player values:
-/// * `set_button(&mut self, button: NesButton, player: Player, pressed: bool)`
-/// * `with_button(self, button: NesButton, player: Player, pressed: bool) -> Self`
-///
-/// `inputs_ident` is optional. If it is omitted then no inputs struct will be defined.
-///
-/// All fields and methods will have `pub` visibility.
-///
-/// All enums and structs will implement the following traits: `Debug`, `Clone`, `Copy`, `PartialEq`,
-/// `Eq`, `Default`, `Hash`, `bincode::Encode`, and `bincode::Decode`. The button enum will additionally
-/// implement the traits `Display` and `FromStr`, and it will have an `ALL` associated constant
-/// containing all enum values.
-#[proc_macro]
-pub fn define_controller_inputs(input: TokenStream) -> TokenStream {
-    inputs::define_controller_inputs(input)
-}
