@@ -6,6 +6,8 @@ use jgenesis_common::num::GetBit;
 use jgenesis_proc_macros::EnumAll;
 use std::fmt::{Display, Formatter};
 
+pub use instructions::cycles_if_move_or_btst;
+
 #[derive(Debug, Clone, Copy)]
 #[cfg_attr(feature = "bincode", derive(bincode::Encode, bincode::Decode))]
 struct ConditionCodes {
@@ -46,6 +48,7 @@ struct Registers {
     usp: u32,
     ssp: u32,
     pc: u32,
+    prefetch: u16,
     ccr: ConditionCodes,
     interrupt_priority_mask: u8,
     pending_interrupt_level: Option<u8>,
@@ -66,6 +69,7 @@ impl Registers {
             usp: 0,
             ssp: 0,
             pc: 0,
+            prefetch: 0,
             ccr: 0.into(),
             interrupt_priority_mask: DEFAULT_INTERRUPT_MASK,
             pending_interrupt_level: None,
@@ -472,7 +476,8 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
 
     // Fetch a word from the current PC and increment PC; returns an address error if PC is odd
     fn fetch_operand(&mut self) -> ExecuteResult<u16> {
-        let operand = self.read_bus_word(self.registers.pc)?;
+        let operand = self.registers.prefetch;
+        self.registers.prefetch = self.read_bus_word(self.registers.pc.wrapping_add(2))?;
         self.registers.pc = self.registers.pc.wrapping_add(2);
 
         Ok(operand)
@@ -851,7 +856,8 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
         self.push_stack_u32(pc)?;
         self.push_stack_u16(sr)?;
 
-        self.registers.pc = self.bus.read_long_word(vector * 4);
+        let new_pc = self.bus.read_long_word(vector * 4);
+        self.jump_to_address(new_pc)?;
 
         Ok(())
     }
@@ -866,11 +872,26 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
         self.push_stack_u16(sr)?;
 
         let vector_addr = AUTO_VECTORED_INTERRUPT_BASE_ADDRESS + 4 * u32::from(interrupt_level);
-        self.registers.pc = self.bus.read_long_word(vector_addr);
+        let new_pc = self.bus.read_long_word(vector_addr);
+        self.jump_to_address(new_pc)?;
 
-        // Auto-vectored interrupt handling takes 44 cycles, but 10 already elapsed from waiting to
-        // acknowledge
-        Ok(34)
+        // Auto-vectored interrupt handling takes 49-59 cycles instead of 44:
+        //   https://gendev.spritesmind.net/forum/viewtopic.php?t=2202
+        // For simplicity, use a constant 54 instead of tracking and synchronizing with E clock.
+        // Return 44 here because 10 cycles have already elapsed prior to the interrupt acknowledge
+        Ok(44)
+    }
+
+    fn jump_to_address(&mut self, address: u32) -> ExecuteResult<()> {
+        self.registers.pc = address.wrapping_sub(2);
+
+        if address % 2 != 0 {
+            return Err(Exception::AddressError(address, BusOpType::Jump));
+        }
+
+        let _ = self.fetch_operand();
+
+        Ok(())
     }
 
     fn execute(mut self) -> u32 {
@@ -904,7 +925,13 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
 
         match self.do_execute() {
             Ok(cycles) => cycles,
-            Err(Exception::AddressError(address, op_type)) => {
+            Err(exception) => self.handle_exception(exception),
+        }
+    }
+
+    fn handle_exception(&mut self, exception: Exception) -> u32 {
+        match exception {
+            Exception::AddressError(address, op_type) => {
                 log::error!(
                     "[{}] Encountered 68000 address error; address={address:08X}, op_type={op_type:?}",
                     self.name
@@ -918,8 +945,8 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
                 // Not completely accurate but close enough; this shouldn't occur in real software
                 50
             }
-            Err(Exception::PrivilegeViolation) => todo!("privilege violation"),
-            Err(Exception::IllegalInstruction(opcode)) => {
+            Exception::PrivilegeViolation => todo!("privilege violation"),
+            Exception::IllegalInstruction(opcode) => {
                 // If the highest 4 bits of the opcode are 1010 or 1111, the CPU uses different
                 // exception vectors. Zaxxon's Motherbase 2000 (32X) depends on this
                 let vector = match opcode >> 12 {
@@ -941,7 +968,7 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
                 // TODO this shouldn't happen in real software
                 34
             }
-            Err(Exception::DivisionByZero { cycles }) => {
+            Exception::DivisionByZero { cycles } => {
                 log::warn!("[{}] Encountered 68000 divide by zero exception", self.name);
 
                 if self.handle_trap(DIVIDE_BY_ZERO_VECTOR, self.registers.pc).is_err() {
@@ -950,14 +977,14 @@ impl<'registers, 'bus, B: BusInterface> InstructionExecutor<'registers, 'bus, B>
 
                 38 + cycles
             }
-            Err(Exception::Trap(vector)) => {
+            Exception::Trap(vector) => {
                 if self.handle_trap(vector, self.registers.pc).is_err() {
                     todo!("address error triggered while executing TRAP instruction")
                 }
 
                 34
             }
-            Err(Exception::CheckRegister { cycles }) => {
+            Exception::CheckRegister { cycles } => {
                 if self.handle_trap(CHECK_REGISTER_VECTOR, self.registers.pc).is_err() {
                     todo!("address error triggered while executing CHK instruction")
                 }
@@ -1033,7 +1060,7 @@ impl M68000 {
         M68000Builder::default()
     }
 
-    fn reset<B: BusInterface>(&mut self, bus: &mut B) {
+    fn reset(&mut self, bus: &mut impl BusInterface) {
         // Reset the upper word of the status register
         self.registers.supervisor_mode = true;
         self.registers.trace_enabled = false;
@@ -1044,6 +1071,18 @@ impl M68000 {
         // Read SSP from $000000 and PC from $000004
         self.registers.ssp = bus.read_long_word(0);
         self.registers.pc = bus.read_long_word(4);
+
+        log::trace!("RESET vector: {:04X}", self.registers.pc);
+
+        self.populate_prefetch(bus);
+    }
+
+    fn populate_prefetch(&mut self, bus: &mut impl BusInterface) {
+        let mut executor =
+            InstructionExecutor::new(&mut self.registers, bus, self.allow_tas_writes, &self.name);
+        if let Err(exception) = executor.jump_to_address(executor.registers.pc) {
+            executor.handle_exception(exception);
+        }
     }
 
     #[must_use]
@@ -1094,8 +1133,9 @@ impl M68000 {
         self.registers.pc
     }
 
-    pub fn set_pc(&mut self, pc: u32) {
+    pub fn set_pc(&mut self, pc: u32, bus: &mut impl BusInterface) {
         self.registers.pc = pc;
+        self.populate_prefetch(bus);
     }
 
     #[must_use]
@@ -1110,6 +1150,12 @@ impl M68000 {
         self.registers.last_instruction_was_muldiv
     }
 
+    #[inline]
+    #[must_use]
+    pub fn next_opcode(&self) -> u16 {
+        self.registers.prefetch
+    }
+
     #[inline]
     pub fn execute_instruction<B: BusInterface>(&mut self, bus: &mut B) -> u32 {
         if bus.reset() {
