@@ -3,5 +3,5 @@ pub mod bus;
 mod core;
 pub mod traits;
 
-pub use crate::core::M68000;
+pub use crate::core::{M68000, cycles_if_move_or_btst};
 pub use traits::BusInterface;
