@@ -2,26 +2,24 @@
 
 use clap::Parser;
 use env_logger::Env;
-use gb_core::api::{GbAspectRatio, GbAudioResampler, GbPalette, GbcColorCorrection};
-use genesis_core::{
-    GenesisAspectRatio, GenesisControllerType, GenesisLowPassFilter, GenesisRegion,
-};
+use gb_config::{GbAspectRatio, GbAudioResampler, GbPalette, GbcColorCorrection};
+use genesis_config::PcmInterpolation;
+use genesis_config::S32XVideoOut;
+use genesis_config::{GenesisAspectRatio, GenesisControllerType, GenesisRegion, Opn2BusyBehavior};
 use jgenesis_common::frontend::{EmulatorTrait, TimingMode};
 use jgenesis_native_config::AppConfig;
-use jgenesis_native_config::common::ConfigSavePath;
-use jgenesis_native_driver::config::input::{NesControllerType, SnesControllerType};
-use jgenesis_native_driver::config::{FullscreenMode, HideMouseCursor};
-use jgenesis_native_driver::{NativeEmulator, NativeTickEffect, extensions};
+use jgenesis_native_config::common::{ConfigSavePath, FullscreenMode, HideMouseCursor};
+use jgenesis_native_config::input::mappings::{NesControllerType, SnesControllerType};
+use jgenesis_native_driver::config::AppConfigExt;
+use jgenesis_native_driver::extensions::{Console, ConsoleWithSize};
+use jgenesis_native_driver::{NativeEmulator, NativeTickEffect};
 use jgenesis_proc_macros::{CustomValueEnum, EnumAll, EnumDisplay};
 use jgenesis_renderer::config::{
     FilterMode, PreprocessShader, PrescaleFactor, Scanlines, VSyncMode, WgpuBackend,
 };
-use nes_core::api::{NesAspectRatio, NesAudioResampler};
-use s32x_core::api::S32XVideoOut;
-use segacd_core::api::{PcmInterpolation, PcmLowPassFilter};
-use smsgg_core::psg::Sn76489Version;
-use smsgg_core::{GgAspectRatio, SmsAspectRatio, SmsModel, SmsRegion};
-use snes_core::api::{AudioInterpolationMode, SnesAspectRatio};
+use nes_config::{NesAspectRatio, NesAudioResampler};
+use smsgg_config::{GgAspectRatio, SmsAspectRatio, SmsGgRegion, SmsModel, Sn76489Version};
+use snes_config::{AudioInterpolationMode, SnesAspectRatio};
 use std::fmt::Debug;
 use std::fs;
 use std::num::{NonZeroU16, NonZeroU32, NonZeroU64};
@@ -111,9 +109,9 @@ struct Args {
     #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
     gg_aspect_ratio: Option<GgAspectRatio>,
 
-    /// Master System region
+    /// Master System / Game Gear region
     #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
-    sms_region: Option<SmsRegion>,
+    smsgg_region: Option<SmsGgRegion>,
 
     /// Crop SMS top and bottom border; almost all games display only the background color in this area
     #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
@@ -136,9 +134,21 @@ struct Args {
     #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
     smsgg_z80_divider: Option<NonZeroU32>,
 
-    /// Emulate the VDP's non-linear DAC, which tends to brighten darker colors and darken brighter colors
+    /// Boot from the SMS BIOS
+    #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
+    sms_boot_from_bios: Option<bool>,
+
+    /// Boot without a cartridge
+    #[arg(long, default_value_t, help_heading = SMSGG_OPTIONS_HEADING)]
+    sms_no_cartridge: bool,
+
+    /// Master System BIOS path
+    #[arg(long, help_heading = SMSGG_OPTIONS_HEADING)]
+    sms_bios_path: Option<PathBuf>,
+
+    /// Emulate the VDP's non-linear color scale, which tends to brighten darker colors and darken brighter colors
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
-    emulate_non_linear_vdp_dac: Option<bool>,
+    genesis_non_linear_color_scale: Option<bool>,
 
     /// Deinterlace if a game enables an interlacing screen mode
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
@@ -165,14 +175,22 @@ struct Args {
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
     emulate_ym2612_ladder_effect: Option<bool>,
 
-    /// Audio low-pass filter setting
+    /// Select OPN2 busy flag behavior
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
-    genesis_low_pass: Option<GenesisLowPassFilter>,
+    opn2_busy_behavior: Option<Opn2BusyBehavior>,
 
     /// Enable audio from the YM2612 FM chip
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
     ym2612_enabled: Option<bool>,
 
+    /// Enable Genesis first-order low-pass filter
+    #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
+    genesis_lpf_enabled: Option<bool>,
+
+    /// Specify Genesis low-pass filter cutoff frequency
+    #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
+    genesis_lpf_cutoff: Option<u32>,
+
     /// Enable audio from the SN76489 PSG chip
     #[arg(long, help_heading = GENESIS_OPTIONS_HEADING)]
     genesis_psg_enabled: Option<bool>,
@@ -226,9 +244,13 @@ struct Args {
     #[arg(long, help_heading = SCD_OPTIONS_HEADING)]
     scd_load_disc_into_ram: Option<bool>,
 
-    /// PCM chip low-pass filter setting
+    /// Enable second-order low-pass filter for PCM chip output
     #[arg(long, help_heading = SCD_OPTIONS_HEADING)]
-    scd_pcm_low_pass: Option<PcmLowPassFilter>,
+    scd_pcm_lpf_enabled: Option<bool>,
+
+    /// Specify PCM chip low-pass filter cutoff frequency
+    #[arg(long, help_heading = SCD_OPTIONS_HEADING)]
+    scd_pcm_lpf_cutoff: Option<u32>,
 
     /// Whether to apply the Genesis low-pass filter to PCM chip output
     #[arg(long, help_heading = SCD_OPTIONS_HEADING)]
@@ -434,6 +456,10 @@ struct Args {
     #[arg(long, help_heading = VIDEO_OPTIONS_HEADING)]
     preprocess_shader: Option<PreprocessShader>,
 
+    /// Mute all audio output
+    #[arg(long, help_heading = AUDIO_OPTIONS_HEADING)]
+    mute_audio: Option<bool>,
+
     /// Audio output frequency (48000 recommended)
     #[arg(long, help_heading = AUDIO_OPTIONS_HEADING)]
     audio_output_frequency: Option<u64>,
@@ -560,30 +586,40 @@ impl Args {
             sms_model,
             sms_aspect_ratio,
             gg_aspect_ratio,
-            sms_region,
             sms_crop_vertical_border,
             sms_crop_left_border,
             gg_use_sms_resolution,
             sms_fm_unit_enabled -> fm_sound_unit_enabled,
             smsgg_z80_divider -> z80_divider,
+            sms_boot_from_bios -> boot_from_bios,
         ]);
 
+        if let Some(region) = self.smsgg_region {
+            config.smsgg.forced_region = Some(region);
+        }
+
         if let Some(psg_version) = self.psg_version {
             config.smsgg.psg_version = Some(psg_version);
         }
+
+        if let Some(bios_path) = &self.sms_bios_path {
+            config.smsgg.bios_path = Some(bios_path.clone());
+        }
     }
 
     fn apply_genesis_overrides(&self, config: &mut AppConfig) {
         apply_overrides!(self, config.genesis, [
-            emulate_non_linear_vdp_dac,
+            genesis_non_linear_color_scale -> non_linear_color_scale,
             genesis_deinterlace -> deinterlace,
             m68k_clock_divider,
             genesis_render_vertical_border -> render_vertical_border,
             genesis_render_horizontal_border -> render_horizontal_border,
             quantize_ym2612_output,
             emulate_ym2612_ladder_effect,
-            genesis_low_pass -> low_pass,
+            opn2_busy_behavior,
             ym2612_enabled,
+            genesis_lpf_enabled,
+            genesis_lpf_cutoff,
             genesis_psg_enabled -> psg_enabled,
             genesis_aspect_ratio -> aspect_ratio,
             genesis_adjust_aspect_ratio -> adjust_aspect_ratio_in_2x_resolution,
@@ -608,7 +644,8 @@ impl Args {
             scd_load_disc_into_ram -> load_disc_into_ram,
             scd_drive_speed -> disc_drive_speed,
             scd_sub_cpu_divider -> sub_cpu_divider,
-            scd_pcm_low_pass -> pcm_low_pass,
+            scd_pcm_lpf_enabled -> pcm_lpf_enabled,
+            scd_pcm_lpf_cutoff -> pcm_lpf_cutoff,
             scd_apply_gen_lpf_to_pcm -> apply_genesis_lpf_to_pcm,
             scd_apply_gen_lpf_to_cd_da -> apply_genesis_lpf_to_cd_da,
             scd_pcm_enabled -> pcm_enabled,
@@ -734,6 +771,7 @@ impl Args {
             self,
             config.common,
             [
+                mute_audio,
                 audio_output_frequency,
                 audio_sync,
                 audio_dynamic_resampling_ratio,
@@ -781,7 +819,7 @@ fn main() -> anyhow::Result<()> {
 
     let hardware = match args.hardware {
         Some(hardware) => hardware,
-        None => guess_hardware(&args)?,
+        None => guess_hardware(&args),
     };
 
     log::info!("Running with hardware {hardware}");
@@ -833,65 +871,35 @@ fn main() -> anyhow::Result<()> {
     }
 }
 
-fn guess_hardware(args: &Args) -> anyhow::Result<Hardware> {
+fn guess_hardware(args: &Args) -> Hardware {
     let file_path = Path::new(&args.file_path);
 
-    let mut file_ext = extensions::from_path(file_path).unwrap_or_default();
+    let console = Console::from_file(file_path)
+        .map(|ConsoleWithSize { console, .. }| console)
+        .unwrap_or_else(|| {
+            log::warn!(
+                "Unable to auto-detect hardware of file at '{}'; defaulting to Genesis",
+                file_path.display()
+            );
+            Console::Genesis
+        });
 
-    match file_ext.as_str() {
-        "zip" => {
-            let zip_entry = jgenesis_native_driver::archive::first_supported_file_in_zip(
-                file_path,
-                &extensions::ALL_CARTRIDGE_BASED,
-            )?
-            .unwrap_or_else(|| {
-                panic!(
-                    "No files with supported extensions found in .zip archive: {}",
-                    args.file_path.display()
-                )
-            });
-            file_ext = zip_entry.extension;
-        }
-        "7z" => {
-            let zip_entry = jgenesis_native_driver::archive::first_supported_file_in_7z(
-                file_path,
-                &extensions::ALL_CARTRIDGE_BASED,
-            )?
-            .unwrap_or_else(|| {
-                panic!(
-                    "No files with supported extensions found in .7z archive: {}",
-                    args.file_path.display()
-                )
-            });
-            file_ext = zip_entry.extension;
-        }
-        _ => {}
+    match console {
+        Console::MasterSystem | Console::GameGear => Hardware::MasterSystem,
+        Console::Genesis => Hardware::Genesis,
+        Console::SegaCd => Hardware::SegaCd,
+        Console::Sega32X => Hardware::Sega32X,
+        Console::Nes => Hardware::Nes,
+        Console::Snes => Hardware::Snes,
+        Console::GameBoy | Console::GameBoyColor => Hardware::GameBoy,
     }
-
-    let file_ext_str = file_ext.as_str();
-    Ok(if extensions::SMSGG.contains(&file_ext_str) {
-        Hardware::MasterSystem
-    } else if extensions::GENESIS.contains(&file_ext_str) {
-        Hardware::Genesis
-    } else if extensions::SEGA_CD.contains(&file_ext_str) {
-        Hardware::SegaCd
-    } else if extensions::SEGA_32X.contains(&file_ext_str) {
-        Hardware::Sega32X
-    } else if extensions::NES.contains(&file_ext_str) {
-        Hardware::Nes
-    } else if extensions::SNES.contains(&file_ext_str) {
-        Hardware::Snes
-    } else if extensions::GB_GBC.contains(&file_ext_str) {
-        Hardware::GameBoy
-    } else {
-        log::warn!("Unrecognized file extension: '{file_ext}' defaulting to Genesis");
-        Hardware::Genesis
-    })
 }
 
 fn run_sms(args: Args, config: AppConfig) -> anyhow::Result<()> {
-    let mut emulator =
-        jgenesis_native_driver::create_smsgg(config.smsgg_config(args.file_path.clone()))?;
+    let mut smsgg_config = config.smsgg_config(args.file_path.clone());
+    smsgg_config.run_without_cartridge = args.sms_no_cartridge;
+
+    let mut emulator = jgenesis_native_driver::create_smsgg(smsgg_config)?;
     run_emulator(&mut emulator, &args)
 }
 
