@@ -16,38 +16,31 @@ use crate::emuthread::{EmuThreadCommand, EmuThreadHandle, EmuThreadStatus};
 use crate::widgets::SavePathSelect;
 use eframe::Frame;
 use egui::panel::TopBottomSide;
+use egui::scroll_area::ScrollAreaOutput;
+use egui::style::ScrollStyle;
 use egui::{
     Align, Button, CentralPanel, Color32, Context, Grid, Key, KeyboardShortcut, Layout, Modifiers,
-    Response, TextEdit, ThemePreference, TopBottomPanel, Ui, Vec2, ViewportCommand, Widget, Window,
-    menu,
+    Response, ScrollArea, TextEdit, ThemePreference, TopBottomPanel, Ui, Vec2, ViewportCommand,
+    Widget, Window, menu,
 };
 use egui_extras::{Column, TableBuilder};
+use emath::Pos2;
+use jgenesis_native_config::common::HideMouseCursor;
 use jgenesis_native_config::{AppConfig, EguiTheme, ListFilters, RecentOpen};
-use jgenesis_native_driver::config::HideMouseCursor;
+use jgenesis_native_driver::extensions::Console;
 use jgenesis_native_driver::{NativeEmulatorError, extensions};
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
 use jgenesis_renderer::config::Scanlines;
 use rfd::FileDialog;
 use std::collections::{HashMap, HashSet};
-use std::fs;
 use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::str::FromStr;
 use std::sync::{Arc, Mutex};
+use std::time::{Duration, Instant};
+use std::{fs, thread};
 use time::{OffsetDateTime, UtcOffset, format_description};
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, EnumAll, EnumDisplay, EnumFromStr)]
-pub enum Console {
-    MasterSystem,
-    GameGear,
-    Genesis,
-    SegaCd,
-    Sega32X,
-    Nes,
-    Snes,
-    GameBoy,
-    GameBoyColor,
-}
+const RESERVED_HELP_TEXT_HEIGHT: f32 = 150.0;
 
 trait ListFiltersExt {
     fn to_console_vec(&self) -> Vec<Console>;
@@ -70,7 +63,7 @@ impl ListFiltersExt for ListFilters {
             self.nes.then_some(Console::Nes),
             self.snes.then_some(Console::Snes),
             self.game_boy.then_some(Console::GameBoy),
-            self.game_boy.then_some(Console::GameBoyColor),
+            self.game_boy_color.then_some(Console::GameBoyColor),
         ]
         .into_iter()
         .flatten()
@@ -330,6 +323,12 @@ impl App {
         self.launch_emulator(path, console);
     }
 
+    fn open_most_recent_file(&mut self) {
+        let Some(recent_open) = self.state.recent_open_list.first() else { return };
+
+        self.launch_emulator(recent_open.full_path.clone(), Some(recent_open.console));
+    }
+
     fn launch_emulator(&mut self, path: PathBuf, console: Option<Console>) {
         self.state.current_file_path.clone_from(&path);
 
@@ -509,6 +508,11 @@ impl App {
             self.open_file(None);
         }
 
+        let open_most_recent_shortcut = KeyboardShortcut::new(Modifiers::NONE, Key::F5);
+        if ctx.input_mut(|input| input.consume_shortcut(&open_most_recent_shortcut)) {
+            self.open_most_recent_file();
+        }
+
         let quit_shortcut = KeyboardShortcut::new(Modifiers::CTRL, Key::Q);
         if ctx.input_mut(|input| input.consume_shortcut(&quit_shortcut)) {
             ctx.send_viewport_cmd(ViewportCommand::Close);
@@ -533,9 +537,26 @@ impl App {
 
                         ui.add_space(5.0);
                     }
+
+                    ui.separator();
+
+                    if ui.button("Clear List").clicked() {
+                        self.config.recent_open_list.clear();
+                        self.state.recent_open_list.clear();
+                        ui.close_menu();
+                    }
                 });
+
+                let open_most_recent_button = Button::new("Open Most Recent")
+                    .shortcut_text(ctx.format_shortcut(&open_most_recent_shortcut));
+                if ui.add(open_most_recent_button).clicked() {
+                    self.open_most_recent_file();
+                    ui.close_menu();
+                }
             });
 
+            ui.add_space(10.0);
+
             ui.menu_button("Open Using", |ui| {
                 for console in [
                     Console::MasterSystem,
@@ -552,6 +573,26 @@ impl App {
 
             ui.add_space(10.0);
 
+            ui.menu_button("Run BIOS", |ui| {
+                for (label, console, has_bios) in [
+                    ("Master System", Console::MasterSystem, self.config.smsgg.bios_path.is_some()),
+                    ("Sega CD", Console::SegaCd, self.config.sega_cd.bios_path.is_some()),
+                ] {
+                    ui.add_enabled_ui(has_bios, |ui| {
+                        if ui.button(label).clicked() {
+                            self.emu_thread.stop_emulator_if_running();
+                            self.emu_thread.send(EmuThreadCommand::RunBios {
+                                console,
+                                config: Box::new(self.config.clone()),
+                            });
+                            ui.close_menu();
+                        }
+                    });
+                }
+            });
+
+            ui.add_space(10.0);
+
             let open_button =
                 Button::new("Open").shortcut_text(ctx.format_shortcut(&open_shortcut));
             if open_button.ui(ui).clicked() {
@@ -986,6 +1027,7 @@ impl App {
             ui.checkbox(&mut self.config.list_filters.nes, "NES");
             ui.checkbox(&mut self.config.list_filters.snes, "SNES");
             ui.checkbox(&mut self.config.list_filters.game_boy, "GB");
+            ui.checkbox(&mut self.config.list_filters.game_boy_color, "GBC");
 
             if prev_list_filters != self.config.list_filters {
                 self.refresh_filtered_rom_list();
@@ -1032,16 +1074,20 @@ impl App {
     }
 
     fn render_help_text(&mut self, ui: &mut Ui, window: OpenWindow) {
-        let Some(help_text) = self.state.help_text.get(&window) else { return };
-
         ui.separator();
 
-        ui.heading(help_text.heading);
+        ui.scope(|ui| {
+            ui.set_min_size([0.0, RESERVED_HELP_TEXT_HEIGHT].into());
 
-        for text in help_text.text {
-            ui.add_space(7.0);
-            ui.label(*text);
-        }
+            let Some(help_text) = self.state.help_text.get(&window) else { return };
+
+            ui.heading(help_text.heading);
+
+            for text in help_text.text {
+                ui.add_space(7.0);
+                ui.label(*text);
+            }
+        });
     }
 
     fn check_emulator_error(&mut self, ctx: &Context) {
@@ -1052,6 +1098,7 @@ impl App {
         if let Some(err) = error_lock.as_ref() {
             let mut open = true;
             match err {
+                NativeEmulatorError::SmsNoBios => self.render_sms_bios_error(ctx, &mut open),
                 NativeEmulatorError::SegaCdNoBios => self.render_scd_bios_error(ctx, &mut open),
                 NativeEmulatorError::SnesLoad(snes_load_err) => {
                     match self.render_snes_load_error(ctx, snes_load_err, &mut open) {
@@ -1129,6 +1176,32 @@ impl App {
             .collect::<Vec<_>>()
             .into();
     }
+
+    fn terminate_emu_thread(&self) {
+        if self.emu_thread.status() == EmuThreadStatus::Terminated {
+            return;
+        }
+
+        let _ = self.emu_thread.try_send(EmuThreadCommand::Terminate);
+
+        let wait_limit = Instant::now() + Duration::from_secs(1);
+        while Instant::now() < wait_limit && self.emu_thread.status() != EmuThreadStatus::Terminated
+        {
+            thread::sleep(Duration::from_millis(1));
+        }
+
+        if self.emu_thread.status() != EmuThreadStatus::Terminated {
+            log::warn!("Failed to terminate emulation thread; exiting anyway");
+        }
+    }
+
+    fn update_window_size_in_config(&mut self, ctx: &Context) {
+        ctx.viewport(|vp| {
+            let Pos2 { x: width, y: height } = vp.input.screen_rect.max;
+            self.config.gui_window_width = width;
+            self.config.gui_window_height = height;
+        });
+    }
 }
 
 impl eframe::App for App {
@@ -1168,6 +1241,8 @@ impl eframe::App for App {
 
         self.render_windows(ctx);
 
+        self.update_window_size_in_config(ctx);
+
         if prev_config != self.config {
             self.state.display_scanlines_warning = should_display_scanlines_warning(&self.config);
 
@@ -1183,6 +1258,10 @@ impl eframe::App for App {
 
         self.state.rendered_first_frame = true;
     }
+
+    fn on_exit(&mut self, _gl: Option<&eframe::glow::Context>) {
+        self.terminate_emu_thread();
+    }
 }
 
 fn should_reload_config(prev_config: &AppConfig, new_config: &AppConfig) -> bool {
@@ -1193,6 +1272,8 @@ fn should_reload_config(prev_config: &AppConfig, new_config: &AppConfig) -> bool
         rom_search_dirs: vec![],
         recent_open_list: vec![],
         egui_theme: EguiTheme::default(),
+        gui_window_width: jgenesis_native_config::DEFAULT_GUI_WIDTH,
+        gui_window_height: jgenesis_native_config::DEFAULT_GUI_HEIGHT,
         ..prev_config.clone()
     };
 
@@ -1201,6 +1282,8 @@ fn should_reload_config(prev_config: &AppConfig, new_config: &AppConfig) -> bool
         rom_search_dirs: vec![],
         recent_open_list: vec![],
         egui_theme: EguiTheme::default(),
+        gui_window_width: jgenesis_native_config::DEFAULT_GUI_WIDTH,
+        gui_window_height: jgenesis_native_config::DEFAULT_GUI_HEIGHT,
         ..new_config.clone()
     };
 
@@ -1220,6 +1303,26 @@ fn format_time_nanos(time_nanos: u128) -> Option<String> {
     local_date_time.format(&format).ok()
 }
 
+fn render_vertical_scroll_area<R>(
+    ui: &mut Ui,
+    add_contents: impl FnOnce(&mut Ui) -> R,
+) -> ScrollAreaOutput<R> {
+    let screen_height = ui.input(|i| i.screen_rect.height());
+
+    let mut scroll_area = ScrollArea::vertical().auto_shrink([false, true]);
+
+    let max_scroll_height = screen_height - RESERVED_HELP_TEXT_HEIGHT - 75.0;
+    if max_scroll_height >= 100.0 {
+        scroll_area = scroll_area.max_height(max_scroll_height);
+    }
+
+    ui.scope(|ui| {
+        ui.style_mut().spacing.scroll = ScrollStyle::solid();
+        scroll_area.show(ui, add_contents)
+    })
+    .inner
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
