@@ -1,14 +1,15 @@
 mod helptext;
 
-use crate::app::{App, Console, OpenWindow};
+use crate::app::{App, Console, OpenWindow, render_vertical_scroll_area};
 use crate::emuthread::EmuThreadStatus;
 use crate::widgets::OverclockSlider;
-use egui::{Context, Window};
-use genesis_core::{GenesisAspectRatio, GenesisLowPassFilter, GenesisRegion};
+use egui::style::ScrollStyle;
+use egui::{Context, Slider, Ui, Window};
+use genesis_config::PcmInterpolation;
+use genesis_config::S32XVideoOut;
+use genesis_config::{GenesisAspectRatio, GenesisRegion, Opn2BusyBehavior};
 use jgenesis_common::frontend::TimingMode;
 use rfd::FileDialog;
-use s32x_core::api::S32XVideoOut;
-use segacd_core::api::{PcmInterpolation, PcmLowPassFilter};
 use std::num::{NonZeroU16, NonZeroU64};
 use std::path::PathBuf;
 
@@ -218,6 +219,11 @@ impl App {
                     ui.label("Aspect ratio");
 
                     ui.horizontal(|ui| {
+                        ui.radio_value(
+                            &mut self.config.genesis.aspect_ratio,
+                            GenesisAspectRatio::Auto,
+                            "Auto",
+                        );
                         ui.radio_value(
                             &mut self.config.genesis.aspect_ratio,
                             GenesisAspectRatio::Ntsc,
@@ -269,6 +275,16 @@ impl App {
                 self.state.help_text.insert(WINDOW, helptext::DOUBLE_SCREEN_INTERLACED_ASPECT);
             }
 
+            let rect = ui
+                .checkbox(
+                    &mut self.config.genesis.non_linear_color_scale,
+                    "Emulate non-linear VDP color scale",
+                )
+                .interact_rect;
+            if ui.rect_contains_pointer(rect) {
+                self.state.help_text.insert(WINDOW, helptext::NON_LINEAR_COLOR_SCALE);
+            }
+
             let rect = ui
                 .checkbox(
                     &mut self.config.genesis.remove_sprite_limits,
@@ -280,17 +296,6 @@ impl App {
                 self.state.help_text.insert(WINDOW, helptext::REMOVE_SPRITE_LIMITS);
             }
 
-            let rect = ui
-                .checkbox(
-                    &mut self.config.genesis.emulate_non_linear_vdp_dac,
-                    "Emulate the VDP's non-linear color DAC",
-                )
-                .on_hover_text("Tends to brighten darker colors and darken brighter colors")
-                .interact_rect;
-            if ui.rect_contains_pointer(rect) {
-                self.state.help_text.insert(WINDOW, helptext::NON_LINEAR_COLOR_DAC);
-            }
-
             let rect = ui
                 .checkbox(&mut self.config.genesis.render_vertical_border, "Render vertical border")
                 .interact_rect;
@@ -369,144 +374,46 @@ impl App {
         const WINDOW: OpenWindow = OpenWindow::GenesisAudio;
 
         let mut open = true;
-        Window::new("Genesis Audio Settings").open(&mut open).resizable(false).show(ctx, |ui| {
-            let rect = ui.checkbox(
-                &mut self.config.genesis.quantize_ym2612_output,
-                "Quantize YM2612 channel output",
-            )
-            .on_hover_text(
-                "Quantize channel outputs from 14 bits to 9 bits to emulate the YM2612's 9-bit DAC",
-            ).interact_rect;
-            if ui.rect_contains_pointer(rect) {
-                self.state.help_text.insert(WINDOW, helptext::QUANTIZE_YM2612_OUTPUT);
-            }
+        Window::new("Genesis Audio Settings").open(&mut open).show(ctx, |ui| {
+            ui.ctx().style_mut(|style| style.spacing.scroll = ScrollStyle::solid());
 
-            let rect = ui
-                .checkbox(
-                    &mut self.config.genesis.emulate_ym2612_ladder_effect,
-                    "Emulate YM2612 DAC distortion (\"ladder effect\")",
-                )
-                .interact_rect;
-            if ui.rect_contains_pointer(rect) {
-                self.state.help_text.insert(WINDOW, helptext::YM2612_LADDER_EFFECT);
-            }
-
-            ui.group(|ui| {
-                ui.label("Low-pass filtering");
-
-                let mut gen_low_pass =
-                    self.config.genesis.low_pass == GenesisLowPassFilter::Model1Va2;
-                let rect = ui
-                    .checkbox(&mut gen_low_pass, "Emulate 3.39 KHz low-pass filter")
-                    .interact_rect;
-                if ui.rect_contains_pointer(rect) {
-                    self.state.help_text.insert(WINDOW, helptext::GENESIS_LOW_PASS);
-                }
-                self.config.genesis.low_pass = if gen_low_pass {
-                    GenesisLowPassFilter::Model1Va2
-                } else {
-                    GenesisLowPassFilter::None
-                };
-
-                let mut pcm_low_pass = self.config.sega_cd.pcm_low_pass == PcmLowPassFilter::SegaCd;
+            render_vertical_scroll_area(ui, |ui| {
                 let rect = ui
                     .checkbox(
-                        &mut pcm_low_pass,
-                        "(Sega CD) Apply 7.97 KHz low-pass filter to PCM chip",
+                        &mut self.config.genesis.quantize_ym2612_output,
+                        "Quantize YM2612 channel output",
+                    )
+                    .on_hover_text("Quantize channel outputs from 14 bits to 9 bits")
+                    .interact_rect;
+                if ui.rect_contains_pointer(rect) {
+                    self.state.help_text.insert(WINDOW, helptext::QUANTIZE_YM2612_OUTPUT);
+                }
+
+                let rect = ui
+                    .checkbox(
+                        &mut self.config.genesis.emulate_ym2612_ladder_effect,
+                        "Emulate YM2612 DAC distortion (\"ladder effect\")",
                     )
                     .interact_rect;
                 if ui.rect_contains_pointer(rect) {
-                    self.state.help_text.insert(WINDOW, helptext::PCM_LOW_PASS);
-                }
-                self.config.sega_cd.pcm_low_pass =
-                    if pcm_low_pass { PcmLowPassFilter::SegaCd } else { PcmLowPassFilter::None };
-
-                let rect = ui
-                    .add_enabled_ui(gen_low_pass, |ui| {
-                        ui.checkbox(
-                            &mut self.config.sega_cd.apply_genesis_lpf_to_pcm,
-                            "(Sega CD) Apply Genesis low-pass filter to PCM chip",
-                        );
-                        ui.checkbox(
-                            &mut self.config.sega_cd.apply_genesis_lpf_to_cd_da,
-                            "(Sega CD) Apply Genesis low-pass filter to CD-DA",
-                        );
-                    })
-                    .response
-                    .interact_rect;
-                if ui.rect_contains_pointer(rect) {
-                    self.state.help_text.insert(WINDOW, helptext::SCD_GEN_LOW_PASS);
+                    self.state.help_text.insert(WINDOW, helptext::YM2612_LADDER_EFFECT);
                 }
 
-                let rect = ui
-                    .add_enabled_ui(gen_low_pass, |ui| {
-                        ui.checkbox(
-                            &mut self.config.sega_32x.apply_genesis_lpf_to_pwm,
-                            "(32X) Apply Genesis low-pass filter to PWM chip",
-                        );
-                    })
-                    .response
-                    .interact_rect;
-                if ui.rect_contains_pointer(rect) {
-                    self.state.help_text.insert(WINDOW, helptext::S32X_GEN_LOW_PASS);
-                }
+                ui.add_space(5.0);
+                ui.group(|ui| {
+                    self.render_low_pass_filter_settings(ui);
+                });
+
+                ui.add_space(5.0);
+                self.render_opn2_busy_flag_setting(ui);
+
+                ui.add_space(5.0);
+                self.render_scd_pcm_interpolation_setting(ui);
+
+                ui.add_space(5.0);
+                self.render_enabled_sound_sources(ui);
             });
 
-            ui.add_space(5.0);
-            let rect = ui
-                .group(|ui| {
-                    ui.label("Sega CD PCM chip interpolation");
-
-                    ui.horizontal(|ui| {
-                        for (value, label) in [
-                            (PcmInterpolation::None, "None"),
-                            (PcmInterpolation::Linear, "Linear"),
-                            (PcmInterpolation::CubicHermite, "4-point Cubic"),
-                            (PcmInterpolation::CubicHermite6Point, "6-point Cubic"),
-                        ] {
-                            ui.radio_value(
-                                &mut self.config.sega_cd.pcm_interpolation,
-                                value,
-                                label,
-                            );
-                        }
-                    });
-                })
-                .response
-                .interact_rect;
-            if ui.rect_contains_pointer(rect) {
-                self.state.help_text.insert(WINDOW, helptext::SCD_PCM_INTERPOLATION);
-            }
-
-            let rect = ui
-                .group(|ui| {
-                    ui.label("Enabled sound sources");
-
-                    ui.horizontal(|ui| {
-                        ui.checkbox(
-                            &mut self.config.genesis.ym2612_enabled,
-                            "YM2612 FM synth chip",
-                        );
-                        ui.checkbox(&mut self.config.genesis.psg_enabled, "SN76489 PSG chip");
-                    });
-                    ui.horizontal(|ui| {
-                        ui.checkbox(
-                            &mut self.config.sega_cd.pcm_enabled,
-                            "(Sega CD) RF5C164 PCM chip",
-                        );
-                        ui.checkbox(
-                            &mut self.config.sega_cd.cd_audio_enabled,
-                            "(Sega CD) CD-DA playback",
-                        );
-                    });
-                    ui.checkbox(&mut self.config.sega_32x.pwm_enabled, "(32X) PWM chip");
-                })
-                .response
-                .interact_rect;
-            if ui.rect_contains_pointer(rect) {
-                self.state.help_text.insert(WINDOW, helptext::SOUND_SOURCES);
-            }
-
             self.render_help_text(ui, WINDOW);
         });
         if !open {
@@ -514,6 +421,222 @@ impl App {
         }
     }
 
+    fn render_low_pass_filter_settings(&mut self, ui: &mut Ui) {
+        const WINDOW: OpenWindow = OpenWindow::GenesisAudio;
+
+        ui.heading("Low-pass filtering");
+
+        let rect = ui
+            .checkbox(
+                &mut self.config.genesis.genesis_lpf_enabled,
+                "Emulate Genesis first-order low-pass filter",
+            )
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::GENESIS_LOW_PASS);
+        }
+
+        let rect = ui
+            .horizontal(|ui| {
+                ui.add_space(15.0);
+
+                ui.label("Cutoff frequency");
+
+                ui.add(
+                    Slider::new(&mut self.config.genesis.genesis_lpf_cutoff, 1..=15000).text("Hz"),
+                );
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::GENESIS_LOW_PASS);
+        }
+
+        let rect = ui
+            .checkbox(
+                &mut self.config.genesis.ym2612_2nd_lpf_enabled,
+                "Emulate YM2612 second-order low-pass filter",
+            )
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::YM2612_2ND_LOW_PASS);
+        }
+
+        let rect = ui
+            .horizontal(|ui| {
+                ui.add_space(15.0);
+
+                ui.label("Cutoff frequency");
+
+                ui.add(
+                    Slider::new(&mut self.config.genesis.ym2612_2nd_lpf_cutoff, 1..=15000)
+                        .text("Hz"),
+                );
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::YM2612_2ND_LOW_PASS);
+        }
+
+        ui.add_space(5.0);
+        ui.horizontal(|ui| {
+            ui.label("Presets:");
+
+            if ui.button("Model 1 VA0-VA2").clicked() {
+                self.config.genesis.genesis_lpf_enabled = true;
+                self.config.genesis.genesis_lpf_cutoff = genesis_config::MODEL_1_VA2_LPF_CUTOFF;
+                self.config.genesis.ym2612_2nd_lpf_enabled = false;
+            }
+
+            if ui.button("Model 1 VA3-VA6").clicked() {
+                self.config.genesis.genesis_lpf_enabled = true;
+                self.config.genesis.genesis_lpf_cutoff = genesis_config::MODEL_1_VA3_LPF_CUTOFF;
+                self.config.genesis.ym2612_2nd_lpf_enabled = false;
+            }
+
+            if ui.button("Model 2").clicked() {
+                self.config.genesis.genesis_lpf_enabled = true;
+                self.config.genesis.genesis_lpf_cutoff = genesis_config::MODEL_2_1ST_LPF_CUTOFF;
+                self.config.genesis.ym2612_2nd_lpf_enabled = true;
+                self.config.genesis.ym2612_2nd_lpf_cutoff = genesis_config::MODEL_2_2ND_LPF_CUTOFF;
+            }
+        });
+
+        ui.separator();
+
+        let rect = ui
+            .checkbox(
+                &mut self.config.sega_cd.pcm_lpf_enabled,
+                "(Sega CD) Apply second-order low-pass filter to PCM chip",
+            )
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::PCM_LOW_PASS);
+        }
+
+        ui.horizontal(|ui| {
+            ui.add_space(15.0);
+
+            ui.label("Cutoff frequency");
+
+            ui.add(Slider::new(&mut self.config.sega_cd.pcm_lpf_cutoff, 1..=15000).text("Hz"));
+
+            if ui.button("Default").clicked() {
+                self.config.sega_cd.pcm_lpf_cutoff = segacd_core::DEFAULT_PCM_LPF_CUTOFF;
+            }
+        });
+
+        let rect = ui
+            .add_enabled_ui(self.config.genesis.genesis_lpf_enabled, |ui| {
+                ui.checkbox(
+                    &mut self.config.sega_cd.apply_genesis_lpf_to_pcm,
+                    "(Sega CD) Apply Genesis low-pass filter to PCM chip",
+                );
+                ui.checkbox(
+                    &mut self.config.sega_cd.apply_genesis_lpf_to_cd_da,
+                    "(Sega CD) Apply Genesis low-pass filter to CD-DA",
+                );
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::SCD_GEN_LOW_PASS);
+        }
+
+        let rect = ui
+            .add_enabled_ui(self.config.genesis.genesis_lpf_enabled, |ui| {
+                ui.checkbox(
+                    &mut self.config.sega_32x.apply_genesis_lpf_to_pwm,
+                    "(32X) Apply Genesis low-pass filter to PWM chip",
+                );
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(WINDOW, helptext::S32X_GEN_LOW_PASS);
+        }
+    }
+
+    fn render_opn2_busy_flag_setting(&mut self, ui: &mut Ui) {
+        let rect = ui
+            .group(|ui| {
+                ui.label("OPN2 busy flag behavior");
+
+                ui.horizontal(|ui| {
+                    ui.radio_value(
+                        &mut self.config.genesis.opn2_busy_behavior,
+                        Opn2BusyBehavior::Ym2612,
+                        "YM2612",
+                    );
+                    ui.radio_value(
+                        &mut self.config.genesis.opn2_busy_behavior,
+                        Opn2BusyBehavior::Ym3438,
+                        "YM3438",
+                    );
+                    ui.radio_value(
+                        &mut self.config.genesis.opn2_busy_behavior,
+                        Opn2BusyBehavior::AlwaysZero,
+                        "Always 0",
+                    );
+                });
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(OpenWindow::GenesisAudio, helptext::OPN2_BUSY_BEHAVIOR);
+        }
+    }
+
+    fn render_scd_pcm_interpolation_setting(&mut self, ui: &mut Ui) {
+        let rect = ui
+            .group(|ui| {
+                ui.label("Sega CD PCM chip interpolation");
+
+                ui.horizontal(|ui| {
+                    for (value, label) in [
+                        (PcmInterpolation::None, "None"),
+                        (PcmInterpolation::Linear, "Linear"),
+                        (PcmInterpolation::CubicHermite, "4-point Cubic"),
+                        (PcmInterpolation::CubicHermite6Point, "6-point Cubic"),
+                    ] {
+                        ui.radio_value(&mut self.config.sega_cd.pcm_interpolation, value, label);
+                    }
+                });
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(OpenWindow::GenesisAudio, helptext::SCD_PCM_INTERPOLATION);
+        }
+    }
+
+    fn render_enabled_sound_sources(&mut self, ui: &mut Ui) {
+        let rect = ui
+            .group(|ui| {
+                ui.label("Enabled sound sources");
+
+                ui.horizontal(|ui| {
+                    ui.checkbox(&mut self.config.genesis.ym2612_enabled, "YM2612 FM synth chip");
+                    ui.checkbox(&mut self.config.genesis.psg_enabled, "SN76489 PSG chip");
+                });
+                ui.horizontal(|ui| {
+                    ui.checkbox(&mut self.config.sega_cd.pcm_enabled, "(Sega CD) RF5C164 PCM chip");
+                    ui.checkbox(
+                        &mut self.config.sega_cd.cd_audio_enabled,
+                        "(Sega CD) CD-DA playback",
+                    );
+                });
+
+                ui.checkbox(&mut self.config.sega_32x.pwm_enabled, "(32X) PWM chip");
+            })
+            .response
+            .interact_rect;
+        if ui.rect_contains_pointer(rect) {
+            self.state.help_text.insert(OpenWindow::GenesisAudio, helptext::SOUND_SOURCES);
+        }
+    }
+
     pub(super) fn render_scd_bios_error(&mut self, ctx: &Context, open: &mut bool) {
         let mut path_configured = false;
         Window::new("Missing Sega CD BIOS").open(open).resizable(false).show(ctx, |ui| {
