@@ -6,7 +6,7 @@ use eframe::emath::Align;
 use eframe::epaint::Color32;
 use egui::{Context, Layout, Window};
 use jgenesis_common::frontend::TimingMode;
-use nes_core::api::{NesAspectRatio, NesAudioResampler, Overscan};
+use nes_config::{NesAspectRatio, NesAudioResampler, Overscan};
 
 pub struct OverscanState {
     top_text: String,
@@ -132,6 +132,16 @@ impl App {
                 self.state.help_text.insert(WINDOW, helptext::ASPECT_RATIO);
             }
 
+            let rect = ui
+                .checkbox(
+                    &mut self.config.nes.ntsc_crop_vertical_overscan,
+                    "(NTSC) Crop vertical overscan",
+                )
+                .interact_rect;
+            if ui.rect_contains_pointer(rect) {
+                self.state.help_text.insert(WINDOW, helptext::NTSC_V_OVERSCAN);
+            }
+
             let rect = ui
                 .checkbox(
                     &mut self.config.nes.remove_sprite_limit,
