@@ -5,9 +5,11 @@ use crate::emuthread::EmuThreadStatus;
 use crate::widgets::OverclockSlider;
 use egui::{Context, Window};
 use jgenesis_common::frontend::TimingMode;
-use smsgg_core::psg::Sn76489Version;
-use smsgg_core::{GgAspectRatio, SmsAspectRatio, SmsModel, SmsRegion};
+use jgenesis_native_driver::extensions::Console;
+use rfd::FileDialog;
+use smsgg_config::{GgAspectRatio, SmsAspectRatio, SmsGgRegion, SmsModel, Sn76489Version};
 use std::num::NonZeroU32;
+use std::path::PathBuf;
 
 impl App {
     pub(super) fn render_smsgg_general_settings(&mut self, ctx: &Context) {
@@ -15,67 +17,114 @@ impl App {
 
         let mut open = true;
         Window::new("SMS/GG General Settings").open(&mut open).resizable(false).show(ctx, |ui| {
-            let rect = ui.group(|ui| {
-                ui.label("Master System timing / display mode");
+            let rect = ui
+                .group(|ui| {
+                    ui.label("Master System timing / display mode");
 
-                ui.horizontal(|ui| {
-                    ui.radio_value(
-                        &mut self.config.smsgg.sms_timing_mode,
-                        TimingMode::Ntsc,
-                        "NTSC (60Hz)",
-                    );
-                    ui.radio_value(&mut self.config.smsgg.sms_timing_mode, TimingMode::Pal, "PAL (50Hz)");
-                });
-            }).response.interact_rect;
+                    ui.horizontal(|ui| {
+                        ui.radio_value(
+                            &mut self.config.smsgg.sms_timing_mode,
+                            TimingMode::Ntsc,
+                            "NTSC (60Hz)",
+                        );
+                        ui.radio_value(
+                            &mut self.config.smsgg.sms_timing_mode,
+                            TimingMode::Pal,
+                            "PAL (50Hz)",
+                        );
+                    });
+                })
+                .response
+                .interact_rect;
             if ui.rect_contains_pointer(rect) {
                 self.state.help_text.insert(WINDOW, helptext::TIMING_MODE);
             }
 
-            let rect = ui.group(|ui| {
-                ui.label("Master System VDP version");
+            let rect = ui
+                .group(|ui| {
+                    ui.label("Master System VDP version");
 
-                ui.horizontal(|ui| {
-                    ui.radio_value(&mut self.config.smsgg.sms_model, SmsModel::Sms1, "SMS1")
-                        .on_hover_text("Emulates an SMS1 quirk that is required for the Japanese version of Ys");
+                    ui.horizontal(|ui| {
+                        ui.radio_value(&mut self.config.smsgg.sms_model, SmsModel::Sms1, "SMS1")
+                            .on_hover_text(concat!(
+                                "Emulates an SMS1 quirk that is",
+                                " required for the Japanese version of Ys"
+                            ));
 
-                    ui.radio_value(&mut self.config.smsgg.sms_model, SmsModel::Sms2, "SMS2");
-                });
-            }).response.interact_rect;
+                        ui.radio_value(&mut self.config.smsgg.sms_model, SmsModel::Sms2, "SMS2");
+                    });
+                })
+                .response
+                .interact_rect;
             if ui.rect_contains_pointer(rect) {
                 self.state.help_text.insert(WINDOW, helptext::VDP_VERSION);
             }
 
-            let rect = ui.group(|ui| {
-                ui.label("Master System region");
+            let rect = ui
+                .group(|ui| {
+                    ui.label("Hardware region");
 
-                ui.horizontal(|ui| {
-                    ui.radio_value(
-                        &mut self.config.smsgg.sms_region,
-                        SmsRegion::International,
-                        "International / Overseas",
-                    );
-                    ui.radio_value(
-                        &mut self.config.smsgg.sms_region,
-                        SmsRegion::Domestic,
-                        "Domestic (Japan)",
-                    );
-                });
-            }).response.interact_rect;
+                    ui.horizontal(|ui| {
+                        ui.radio_value(&mut self.config.smsgg.forced_region, None, "Auto");
+                        ui.radio_value(
+                            &mut self.config.smsgg.forced_region,
+                            Some(SmsGgRegion::International),
+                            "International / Overseas",
+                        );
+                        ui.radio_value(
+                            &mut self.config.smsgg.forced_region,
+                            Some(SmsGgRegion::Domestic),
+                            "Domestic (Japan)",
+                        );
+                    });
+                })
+                .response
+                .interact_rect;
             if ui.rect_contains_pointer(rect) {
                 self.state.help_text.insert(WINDOW, helptext::REGION);
             }
 
-            let rect = ui.add(OverclockSlider {
-                label: "Z80 clock divider",
-                current_value: &mut self.config.smsgg.z80_divider,
-                range: NonZeroU32::new(1).unwrap()..=NonZeroU32::new(15).unwrap(),
-                master_clock: smsgg_core::audio::NTSC_MCLK_FREQUENCY,
-                default_divider: smsgg_core::NATIVE_Z80_DIVIDER.into(),
-            }).interact_rect;
+            let rect = ui
+                .add(OverclockSlider {
+                    label: "Z80 clock divider",
+                    current_value: &mut self.config.smsgg.z80_divider,
+                    range: NonZeroU32::new(1).unwrap()..=NonZeroU32::new(15).unwrap(),
+                    master_clock: smsgg_core::audio::NTSC_MCLK_FREQUENCY,
+                    default_divider: smsgg_core::NATIVE_Z80_DIVIDER.into(),
+                })
+                .interact_rect;
             if ui.rect_contains_pointer(rect) {
                 self.state.help_text.insert(WINDOW, helptext::Z80_OVERCLOCK);
             }
 
+            ui.add_space(5.0);
+            let rect = ui
+                .checkbox(&mut self.config.smsgg.boot_from_bios, "(SMS) Boot from BIOS ROM")
+                .interact_rect;
+            if ui.rect_contains_pointer(rect) {
+                self.state.help_text.insert(WINDOW, helptext::SMS_BIOS);
+            }
+
+            let rect = ui
+                .horizontal(|ui| {
+                    let button_label = match &self.config.smsgg.bios_path {
+                        Some(bios_path) => bios_path.to_string_lossy(),
+                        None => "<None>".into(),
+                    };
+                    if ui.button(button_label).clicked() {
+                        if let Some(bios_path) = pick_sms_bios_path() {
+                            self.config.smsgg.bios_path = Some(bios_path);
+                        }
+                    }
+
+                    ui.label("SMS BIOS path");
+                })
+                .response
+                .interact_rect;
+            if ui.rect_contains_pointer(rect) {
+                self.state.help_text.insert(WINDOW, helptext::SMS_BIOS);
+            }
+
             self.render_help_text(ui, WINDOW);
         });
         if !open {
@@ -253,4 +302,32 @@ impl App {
             self.state.open_windows.remove(&WINDOW);
         }
     }
+
+    pub(super) fn render_sms_bios_error(&mut self, ctx: &Context, open: &mut bool) {
+        let mut path_configured = false;
+        Window::new("Missing SMS BIOS").open(open).resizable(false).show(ctx, |ui| {
+            ui.label("The boot from BIOS option is set but no SMS BIOS path is configured.");
+
+            ui.add_space(10.0);
+
+            ui.horizontal(|ui| {
+                ui.label("Configure now:");
+                if ui.button("Configure SMS BIOS path").clicked() {
+                    if let Some(bios_path) = pick_sms_bios_path() {
+                        self.config.smsgg.bios_path = Some(bios_path);
+                        path_configured = true;
+                    }
+                }
+            });
+        });
+
+        if path_configured {
+            *open = false;
+            self.launch_emulator(self.state.current_file_path.clone(), Some(Console::MasterSystem));
+        }
+    }
+}
+
+fn pick_sms_bios_path() -> Option<PathBuf> {
+    FileDialog::new().add_filter("bin", &["sms", "bin"]).add_filter("All Files", &["*"]).pick_file()
 }
