@@ -1,12 +1,12 @@
 mod inputwindow;
 
-use crate::app::Console;
 use crate::emuthread::inputwindow::InputWindow;
 use anyhow::anyhow;
 use jgenesis_native_config::AppConfig;
-use jgenesis_native_driver::input::{
-    AxisDirection, GamepadAction, GenericInput, HatDirection, Joysticks,
-};
+use jgenesis_native_config::input::{AxisDirection, GamepadAction, GenericInput, HatDirection};
+use jgenesis_native_driver::config::AppConfigExt;
+use jgenesis_native_driver::extensions::Console;
+use jgenesis_native_driver::input::Joysticks;
 use jgenesis_native_driver::{
     AudioError, Native32XEmulator, NativeEmulatorError, NativeEmulatorResult,
     NativeGameBoyEmulator, NativeGenesisEmulator, NativeNesEmulator, NativeSegaCdEmulator,
@@ -20,6 +20,7 @@ use segacd_core::api::SegaCdLoadResult;
 use std::collections::HashSet;
 use std::path::PathBuf;
 use std::sync::atomic::{AtomicBool, AtomicU8, Ordering};
+use std::sync::mpsc::SendError;
 use std::sync::mpsc::{Receiver, Sender, TryRecvError};
 use std::sync::{Arc, Mutex, mpsc};
 use std::thread;
@@ -36,6 +37,7 @@ pub enum EmuThreadStatus {
     RunningSnes = 6,
     RunningGameBoy = 7,
     WaitingForFirstCommand = 8,
+    Terminated = 9,
 }
 
 impl EmuThreadStatus {
@@ -50,6 +52,7 @@ impl EmuThreadStatus {
             6 => Self::RunningSnes,
             7 => Self::RunningGameBoy,
             8 => Self::WaitingForFirstCommand,
+            9 => Self::Terminated,
             _ => panic!("invalid status discriminant: {discriminant}"),
         }
     }
@@ -68,7 +71,11 @@ impl EmuThreadStatus {
     }
 }
 
-impl Console {
+trait ConsoleExt {
+    fn running_status(self) -> EmuThreadStatus;
+}
+
+impl ConsoleExt for Console {
     fn running_status(self) -> EmuThreadStatus {
         match self {
             Self::MasterSystem | Self::GameGear => EmuThreadStatus::RunningSmsGg,
@@ -85,8 +92,10 @@ impl Console {
 #[derive(Debug, Clone)]
 pub enum EmuThreadCommand {
     Run { console: Console, config: Box<AppConfig>, file_path: PathBuf },
+    RunBios { console: Console, config: Box<AppConfig> },
     ReloadConfig(Box<AppConfig>, PathBuf),
     StopEmulator,
+    Terminate,
     CollectInput { axis_deadzone: i16 },
     SoftReset,
     HardReset,
@@ -111,6 +120,10 @@ impl EmuThreadHandle {
         self.command_sender.send(command).unwrap();
     }
 
+    pub fn try_send(&self, command: EmuThreadCommand) -> Result<(), SendError<EmuThreadCommand>> {
+        self.command_sender.send(command)
+    }
+
     pub fn status(&self) -> EmuThreadStatus {
         EmuThreadStatus::from_discriminant(self.status.load(Ordering::Relaxed))
     }
@@ -221,7 +234,36 @@ fn thread_run(ctx: EmuThreadContext) {
                         continue;
                     }
                 };
-                run_emulator(emulator, &ctx);
+                let run_result = run_emulator(emulator, &ctx);
+
+                if run_result == RunEmuResult::Terminate {
+                    ctx.status.store(EmuThreadStatus::Terminated as u8, Ordering::Relaxed);
+                    return;
+                }
+            }
+            Ok(EmuThreadCommand::RunBios { console, mut config }) => {
+                ctx.status.store(console.running_status() as u8, Ordering::Relaxed);
+
+                if let Some(native_ppi) = ctx.egui_ctx.native_pixels_per_point() {
+                    log::info!("Setting emulator window scale factor to {native_ppi}");
+                    config.common.window_scale_factor = Some(native_ppi);
+                }
+
+                let emulator = match GenericEmulator::create_run_bios(console, config) {
+                    Ok(Some(emulator)) => emulator,
+                    Ok(None) => continue,
+                    Err(err) => {
+                        log::error!("Error initializing emulator: {err}");
+                        *ctx.emulator_error.lock().unwrap() = Some(err);
+                        continue;
+                    }
+                };
+                let run_result = run_emulator(emulator, &ctx);
+
+                if run_result == RunEmuResult::Terminate {
+                    ctx.status.store(EmuThreadStatus::Terminated as u8, Ordering::Relaxed);
+                    return;
+                }
             }
             Ok(EmuThreadCommand::CollectInput { axis_deadzone }) => {
                 match collect_input_not_running(axis_deadzone, ctx.egui_ctx.pixels_per_point()) {
@@ -234,6 +276,11 @@ fn thread_run(ctx: EmuThreadContext) {
                     }
                 }
             }
+            Ok(EmuThreadCommand::Terminate) => {
+                log::info!("Terminating emulation thread");
+                ctx.status.store(EmuThreadStatus::Terminated as u8, Ordering::Relaxed);
+                return;
+            }
             Ok(
                 EmuThreadCommand::StopEmulator
                 | EmuThreadCommand::ReloadConfig(..)
@@ -257,13 +304,13 @@ fn thread_run(ctx: EmuThreadContext) {
 
 #[derive(MatchEachVariantMacro)]
 enum GenericEmulator {
-    SmsGg(NativeSmsGgEmulator),
-    Genesis(NativeGenesisEmulator),
-    SegaCd(NativeSegaCdEmulator),
-    Sega32X(Native32XEmulator),
-    Nes(NativeNesEmulator),
-    Snes(NativeSnesEmulator),
-    GameBoy(NativeGameBoyEmulator),
+    SmsGg(Box<NativeSmsGgEmulator>),
+    Genesis(Box<NativeGenesisEmulator>),
+    SegaCd(Box<NativeSegaCdEmulator>),
+    Sega32X(Box<Native32XEmulator>),
+    Nes(Box<NativeNesEmulator>),
+    Snes(Box<NativeSnesEmulator>),
+    GameBoy(Box<NativeGameBoyEmulator>),
 }
 
 impl GenericEmulator {
@@ -273,30 +320,56 @@ impl GenericEmulator {
         path: PathBuf,
     ) -> NativeEmulatorResult<Self> {
         let emulator = match console {
-            Console::MasterSystem | Console::GameGear => {
-                Self::SmsGg(jgenesis_native_driver::create_smsgg(config.smsgg_config(path))?)
+            Console::MasterSystem | Console::GameGear => Self::SmsGg(Box::new(
+                jgenesis_native_driver::create_smsgg(config.smsgg_config(path))?,
+            )),
+            Console::Genesis => Self::Genesis(Box::new(jgenesis_native_driver::create_genesis(
+                config.genesis_config(path),
+            )?)),
+            Console::SegaCd => Self::SegaCd(Box::new(jgenesis_native_driver::create_sega_cd(
+                config.sega_cd_config(path),
+            )?)),
+            Console::Sega32X => Self::Sega32X(Box::new(jgenesis_native_driver::create_32x(
+                config.sega_32x_config(path),
+            )?)),
+            Console::Nes => {
+                Self::Nes(Box::new(jgenesis_native_driver::create_nes(config.nes_config(path))?))
             }
-            Console::Genesis => {
-                Self::Genesis(jgenesis_native_driver::create_genesis(config.genesis_config(path))?)
-            }
-            Console::SegaCd => {
-                Self::SegaCd(jgenesis_native_driver::create_sega_cd(config.sega_cd_config(path))?)
-            }
-            Console::Sega32X => {
-                Self::Sega32X(jgenesis_native_driver::create_32x(config.sega_32x_config(path))?)
-            }
-            Console::Nes => Self::Nes(jgenesis_native_driver::create_nes(config.nes_config(path))?),
             Console::Snes => {
-                Self::Snes(jgenesis_native_driver::create_snes(config.snes_config(path))?)
+                Self::Snes(Box::new(jgenesis_native_driver::create_snes(config.snes_config(path))?))
             }
             Console::GameBoy | Console::GameBoyColor => {
-                Self::GameBoy(jgenesis_native_driver::create_gb(config.gb_config(path))?)
+                Self::GameBoy(Box::new(jgenesis_native_driver::create_gb(config.gb_config(path))?))
             }
         };
 
         Ok(emulator)
     }
 
+    fn create_run_bios(
+        console: Console,
+        config: Box<AppConfig>,
+    ) -> NativeEmulatorResult<Option<Self>> {
+        let emulator = match console {
+            Console::MasterSystem => {
+                let mut sms_config = config.smsgg_config(PathBuf::new());
+                sms_config.boot_from_bios = true;
+                sms_config.run_without_cartridge = true;
+
+                Self::SmsGg(Box::new(jgenesis_native_driver::create_smsgg(sms_config)?))
+            }
+            Console::SegaCd => {
+                let mut scd_config = config.sega_cd_config(PathBuf::new());
+                scd_config.run_without_disc = true;
+
+                Self::SegaCd(Box::new(jgenesis_native_driver::create_sega_cd(scd_config)?))
+            }
+            _ => return Ok(None),
+        };
+
+        Ok(Some(emulator))
+    }
+
     fn reload_config(&mut self, config: Box<AppConfig>, path: PathBuf) -> Result<(), AudioError> {
         match self {
             Self::SmsGg(emulator) => emulator.reload_smsgg_config(config.smsgg_config(path)),
@@ -364,7 +437,14 @@ impl GenericEmulator {
     }
 }
 
-fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) {
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+enum RunEmuResult {
+    None,
+    Terminate,
+}
+
+#[must_use]
+fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) -> RunEmuResult {
     loop {
         match emulator.render_frame() {
             Ok(None) => {
@@ -375,12 +455,16 @@ fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) {
                         EmuThreadCommand::ReloadConfig(config, path) => {
                             if let Err(err) = emulator.reload_config(config, path) {
                                 *ctx.emulator_error.lock().unwrap() = Some(err.into());
-                                return;
+                                return RunEmuResult::None;
                             }
                         }
                         EmuThreadCommand::StopEmulator => {
                             log::info!("Stopping emulator");
-                            return;
+                            return RunEmuResult::None;
+                        }
+                        EmuThreadCommand::Terminate => {
+                            log::info!("Terminating emulation thread");
+                            return RunEmuResult::Terminate;
                         }
                         EmuThreadCommand::CollectInput { axis_deadzone } => {
                             log::debug!("Received collect input command");
@@ -397,7 +481,7 @@ fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) {
 
                             if is_none {
                                 // Window was closed
-                                return;
+                                return RunEmuResult::None;
                             }
                         }
                         EmuThreadCommand::SoftReset => emulator.soft_reset(),
@@ -409,24 +493,24 @@ fn run_emulator(mut emulator: GenericEmulator, ctx: &EmuThreadContext) {
                         EmuThreadCommand::SegaCdChangeDisc(path) => {
                             if let Err(err) = emulator.change_disc(path) {
                                 *ctx.emulator_error.lock().unwrap() = Some(err.into());
-                                return;
+                                return RunEmuResult::None;
                             }
                         }
-                        EmuThreadCommand::Run { .. } => {}
+                        EmuThreadCommand::Run { .. } | EmuThreadCommand::RunBios { .. } => {}
                     }
                 }
             }
             Ok(Some(NativeTickEffect::PowerOff)) => {
-                return;
+                return RunEmuResult::None;
             }
             Ok(Some(NativeTickEffect::Exit)) => {
                 ctx.exit_signal.store(true, Ordering::Relaxed);
-                return;
+                return RunEmuResult::Terminate;
             }
             Err(err) => {
                 log::error!("Emulator terminated with an error: {err}");
                 *ctx.emulator_error.lock().unwrap() = Some(err);
-                return;
+                return RunEmuResult::None;
             }
         }
     }
