@@ -3,7 +3,9 @@ use eframe::NativeOptions;
 use egui::{Vec2, ViewportBuilder};
 use env_logger::Env;
 use jgenesis_gui::app::{App, LoadAtStartup};
-use std::path::PathBuf;
+use jgenesis_native_config::AppConfig;
+use std::fs;
+use std::path::{Path, PathBuf};
 
 #[derive(Debug, Parser)]
 struct Args {
@@ -80,10 +82,33 @@ fn steam_deck_dpi_hack() {
         log::info!(
             "It looks like this is a Steam Deck; overriding winit scale factor to 1 as otherwise it will default to 4.5"
         );
-        std::env::set_var("WINIT_X11_SCALE_FACTOR", "1");
+
+        // SAFETY: This function is only called during initialization, before spawning any threads
+        unsafe {
+            std::env::set_var("WINIT_X11_SCALE_FACTOR", "1");
+        }
     }
 }
 
+fn initial_gui_size(config_path: &Path) -> (f32, f32) {
+    let mut gui_width = jgenesis_native_config::DEFAULT_GUI_WIDTH;
+    let mut gui_height = jgenesis_native_config::DEFAULT_GUI_HEIGHT;
+
+    if let Some(config) = fs::read_to_string(config_path)
+        .ok()
+        .and_then(|config_str| toml::from_str::<AppConfig>(&config_str).ok())
+    {
+        gui_width = f32_max(jgenesis_native_config::DEFAULT_GUI_WIDTH, config.gui_window_width);
+        gui_height = f32_max(jgenesis_native_config::DEFAULT_GUI_HEIGHT, config.gui_window_height);
+    }
+
+    (gui_width, gui_height)
+}
+
+fn f32_max(value: f32, max: f32) -> f32 {
+    if value < max { max } else { value }
+}
+
 fn main() -> eframe::Result<()> {
     env_logger::Builder::from_env(
         Env::default().default_filter_or("info,wgpu_core=warn,wgpu_hal=warn"),
@@ -103,8 +128,10 @@ fn main() -> eframe::Result<()> {
         log::info!("Will open file '{}' after starting", file_path.display());
     }
 
+    let (gui_width, gui_height) = initial_gui_size(&config_path);
+
     let options = NativeOptions {
-        viewport: ViewportBuilder::default().with_inner_size(Vec2::new(800.0, 600.0)),
+        viewport: ViewportBuilder::default().with_inner_size(Vec2::new(gui_width, gui_height)),
         ..NativeOptions::default()
     };
 
