@@ -1,4 +1,3 @@
-use crate::config::RomReadResult;
 use crate::extensions;
 use std::fs::File;
 use std::io;
@@ -28,7 +27,9 @@ pub enum ArchiveError {
         #[source]
         source: sevenz_rust::Error,
     },
-    #[error("No supported files found in .zip archive '{path}'")]
+    #[error("File '{file_name}' not found in archive '{path}'")]
+    FileNotFound { path: String, file_name: String },
+    #[error("No files with supported extensions found in archive '{path}'")]
     NoSupportedFiles { path: String },
 }
 
@@ -45,8 +46,8 @@ impl ArchiveError {
         Self::SevenZ { path: path.display().to_string(), source }
     }
 
-    fn no_supported_files(path: &Path) -> Self {
-        Self::NoSupportedFiles { path: path.display().to_string() }
+    fn file_not_found(path: &Path, file_name: &str) -> Self {
+        Self::FileNotFound { path: path.display().to_string(), file_name: file_name.into() }
     }
 }
 
@@ -61,6 +62,138 @@ fn extension_matches(file_name: &str, target_extension: &str) -> bool {
     extensions::from_path(file_name).is_some_and(|file_ext| file_ext.as_str() == target_extension)
 }
 
+#[derive(Debug, Clone, Copy)]
+pub struct ArchiveEntry<'a> {
+    pub file_name: &'a str,
+    pub size: u64,
+}
+
+/// List all files in a .zip archive.
+///
+/// The given callback will be called for every file entry unless there is an error.
+///
+/// # Errors
+///
+/// Propagates any I/O or decoding errors.
+pub fn list_files_zip(
+    zip_path: &Path,
+    mut callback: impl FnMut(ArchiveEntry<'_>),
+) -> Result<(), ArchiveError> {
+    let io_err_fn = |source| ArchiveError::io(zip_path, source);
+    let zip_err_fn = |source| ArchiveError::zip(zip_path, source);
+
+    let file = File::open(zip_path).map_err(io_err_fn)?;
+    let reader = BufReader::new(file);
+    let mut archive = ZipArchive::new(reader).map_err(zip_err_fn)?;
+
+    for i in 0..archive.len() {
+        let entry = archive.by_index(i).map_err(zip_err_fn)?;
+        callback(ArchiveEntry { file_name: entry.name(), size: entry.size() });
+    }
+
+    Ok(())
+}
+
+/// List all files in a .7z archive.
+///
+/// The given callback will be called for every file entry unless there is an error.
+///
+/// # Errors
+///
+/// Propagates any I/O or decoding errors.
+pub fn list_files_7z(
+    sevenz_path: &Path,
+    mut callback: impl FnMut(ArchiveEntry<'_>),
+) -> Result<(), ArchiveError> {
+    let io_err_fn = |source| ArchiveError::io(sevenz_path, source);
+    let sevenz_err_fn = |source| ArchiveError::sevenz(sevenz_path, source);
+
+    let file = File::open(sevenz_path).map_err(io_err_fn)?;
+    let file_len = file.metadata().map_err(io_err_fn)?.len();
+    let mut reader = BufReader::new(file);
+    let archive = sevenz_rust::Archive::read(&mut reader, file_len, &[]).map_err(sevenz_err_fn)?;
+
+    for entry in &archive.files {
+        if !entry.has_stream {
+            // Not a readable file (e.g. is a directory)
+            continue;
+        }
+
+        callback(ArchiveEntry { file_name: entry.name.as_str(), size: entry.size });
+    }
+
+    Ok(())
+}
+
+/// Read a file from within a .zip archive.
+///
+/// # Errors
+///
+/// Propagates any I/O or decoding errors.
+///
+/// Will return an error if the archive does not contain the specified file.
+pub fn read_file_zip(zip_path: &Path, file_name: &str) -> Result<Vec<u8>, ArchiveError> {
+    let io_err_fn = |source| ArchiveError::io(zip_path, source);
+    let zip_err_fn = |source| ArchiveError::zip(zip_path, source);
+
+    let file = File::open(zip_path).map_err(io_err_fn)?;
+    let reader = BufReader::new(file);
+    let mut archive = ZipArchive::new(reader).map_err(zip_err_fn)?;
+
+    let mut entry = archive.by_name(file_name).map_err(zip_err_fn)?;
+
+    let mut buffer = Vec::with_capacity(entry.size() as usize);
+    entry.read_to_end(&mut buffer).map_err(io_err_fn)?;
+
+    Ok(buffer)
+}
+
+/// Read a file from within a .7z archive.
+///
+/// # Errors
+///
+/// Propagates any I/O or decoding errors.
+///
+/// Will return an error if the archive does not contain the specified file.
+pub fn read_file_7z(sevenz_path: &Path, file_name: &str) -> Result<Vec<u8>, ArchiveError> {
+    let io_err_fn = |source| ArchiveError::io(sevenz_path, source);
+    let sevenz_err_fn = |source| ArchiveError::sevenz(sevenz_path, source);
+
+    let file = File::open(sevenz_path).map_err(io_err_fn)?;
+    let file_len = file.metadata().map_err(io_err_fn)?.len();
+    let mut reader = BufReader::new(file);
+    let archive = sevenz_rust::Archive::read(&mut reader, file_len, &[]).map_err(sevenz_err_fn)?;
+
+    for folder_idx in 0..archive.folders.len() {
+        let decoder = sevenz_rust::BlockDecoder::new(folder_idx, &archive, &[], &mut reader);
+
+        if !decoder.entries().iter().any(|entry| entry.name.as_str() == file_name) {
+            continue;
+        }
+
+        let mut buffer = Vec::new();
+        let mut found = false;
+        decoder
+            .for_each_entries(&mut |entry, reader| {
+                if entry.name.as_str() == file_name {
+                    reader.read_to_end(&mut buffer)?;
+                    found = true;
+                    Ok(false)
+                } else {
+                    io::copy(reader, &mut io::sink())?;
+                    Ok(true)
+                }
+            })
+            .map_err(sevenz_err_fn)?;
+
+        if found {
+            return Ok(buffer);
+        }
+    }
+
+    Err(ArchiveError::file_not_found(sevenz_path, file_name))
+}
+
 /// Returns metadata of the first file in the .zip archive that has a supported extension, or
 /// None if there are no files with a supported extension.
 ///
@@ -107,146 +240,3 @@ pub fn first_supported_file_in_zip(
 
     Ok(Some(ZipEntryMetadata { file_name, extension, size }))
 }
-
-/// Returns metadata of the first file in the .7z archive that has a supported extension, or
-/// None if there are no files with a supported extension.
-///
-/// Will also return None if the archive contains any .cue files, under the assumption that the
-/// archive contains a CD-ROM image.
-///
-/// # Errors
-///
-/// Will propagate any I/O or 7ZIP errors.
-pub fn first_supported_file_in_7z(
-    sevenz_path: &Path,
-    supported_extensions: &[&str],
-) -> Result<Option<ZipEntryMetadata>, ArchiveError> {
-    let io_err_fn = |source| ArchiveError::io(sevenz_path, source);
-    let sevenz_err_fn = |source| ArchiveError::sevenz(sevenz_path, source);
-
-    let file = File::open(sevenz_path).map_err(io_err_fn)?;
-    let file_len = file.metadata().map_err(io_err_fn)?.len();
-    let mut reader = BufReader::new(file);
-    let archive = sevenz_rust::Archive::read(&mut reader, file_len, &[]).map_err(sevenz_err_fn)?;
-
-    let mut first_supported_file: Option<ZipEntryMetadata> = None;
-    for folder_idx in 0..archive.folders.len() {
-        let folder_dec = sevenz_rust::BlockDecoder::new(folder_idx, &archive, &[], &mut reader);
-
-        for entry in folder_dec.entries() {
-            if !entry.has_stream {
-                // Is a directory
-                continue;
-            }
-
-            if extension_matches(&entry.name, "cue") {
-                // Archive contains a .cue file; assume it's a CD-ROM image
-                return Ok(None);
-            }
-
-            if first_supported_file.is_some() {
-                // Already found a supported file, only checking for .cue files now
-                continue;
-            }
-
-            for &extension in supported_extensions {
-                if extension_matches(&entry.name, extension) {
-                    first_supported_file = Some(ZipEntryMetadata {
-                        file_name: entry.name.clone(),
-                        extension: extension.to_string(),
-                        size: entry.size,
-                    });
-                    break;
-                }
-            }
-        }
-    }
-
-    Ok(first_supported_file)
-}
-
-/// Opens and reads the first file in the .zip archive that has a supported extension.
-///
-/// # Errors
-///
-/// Propagates any I/O or ZIP errors, and will also return an error if the .zip archive contains
-/// no files with a supported extension.
-pub(crate) fn read_first_file_in_zip(
-    zip_path: &Path,
-    supported_extensions: &[&str],
-) -> Result<RomReadResult, ArchiveError> {
-    let io_err_fn = |source| ArchiveError::io(zip_path, source);
-    let zip_err_fn = |source| ArchiveError::zip(zip_path, source);
-
-    let file = File::open(zip_path).map_err(io_err_fn)?;
-    let reader = BufReader::new(file);
-    let mut archive = ZipArchive::new(reader).map_err(zip_err_fn)?;
-
-    let file_names: Vec<_> = archive.file_names().map(String::from).collect();
-    for file_name in file_names {
-        let Some(extension) = extensions::from_path(&file_name) else {
-            continue;
-        };
-
-        if supported_extensions.contains(&extension.as_str()) {
-            let mut zip_file = archive.by_name(&file_name).map_err(zip_err_fn)?;
-
-            let mut contents = Vec::with_capacity(zip_file.size() as usize);
-            zip_file.read_to_end(&mut contents).map_err(io_err_fn)?;
-
-            return Ok(RomReadResult { rom: contents, extension });
-        }
-    }
-
-    Err(ArchiveError::no_supported_files(zip_path))
-}
-
-pub(crate) fn read_first_file_in_7z(
-    sevenz_path: &Path,
-    supported_extensions: &[&str],
-) -> Result<RomReadResult, ArchiveError> {
-    let io_err_fn = |source| ArchiveError::io(sevenz_path, source);
-    let sevenz_err_fn = |source| ArchiveError::sevenz(sevenz_path, source);
-
-    let file = File::open(sevenz_path).map_err(io_err_fn)?;
-    let file_len = file.metadata().map_err(io_err_fn)?.len();
-    let mut reader = BufReader::new(file);
-    let archive = sevenz_rust::Archive::read(&mut reader, file_len, &[]).map_err(sevenz_err_fn)?;
-
-    for folder_idx in 0..archive.folders.len() {
-        let folder_dec = sevenz_rust::BlockDecoder::new(folder_idx, &archive, &[], &mut reader);
-
-        let Some((file_name, extension)) = folder_dec.entries().iter().find_map(|entry| {
-            if !entry.has_stream {
-                return None;
-            }
-
-            for &extension in supported_extensions {
-                if extension_matches(&entry.name, extension) {
-                    return Some((entry.name.clone(), extension.to_string()));
-                }
-            }
-
-            None
-        }) else {
-            continue;
-        };
-
-        let mut decompressed = Vec::new();
-        folder_dec
-            .for_each_entries(&mut |entry, reader| {
-                if entry.name == file_name {
-                    reader.read_to_end(&mut decompressed)?;
-                    return Ok(false);
-                }
-
-                io::copy(reader, &mut io::sink())?;
-                Ok(true)
-            })
-            .map_err(sevenz_err_fn)?;
-
-        return Ok(RomReadResult { rom: decompressed, extension });
-    }
-
-    Err(ArchiveError::no_supported_files(sevenz_path))
-}
