@@ -1,156 +1,24 @@
-mod serialize;
-
 use arrayvec::ArrayVec;
 use jgenesis_common::frontend::{DisplayArea, FrameSize, MappableInputs};
 use jgenesis_common::input::Player;
-use jgenesis_proc_macros::{EnumAll, EnumDisplay, EnumFromStr};
+use jgenesis_native_config::input::{
+    AxisDirection, GamepadAction, GenericInput, HatDirection, Hotkey,
+};
 use rustc_hash::{FxHashMap, FxHashSet};
 use sdl2::event::{Event, WindowEvent};
 use sdl2::joystick::{HatState, Joystick};
 use sdl2::keyboard::Keycode;
-use sdl2::mouse::MouseButton;
 use sdl2::{IntegerOrSdlError, JoystickSubsystem};
 use std::array;
-use std::borrow::Cow;
 use std::cell::RefCell;
 use std::collections::BTreeMap;
-use std::fmt::{Debug, Display, Formatter};
+use std::fmt::Debug;
 use std::hash::Hash;
 use std::rc::Rc;
-use std::str::FromStr;
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub enum AxisDirection {
-    Positive,
-    Negative,
-}
-
-impl AxisDirection {
-    #[inline]
-    #[must_use]
-    pub fn from_value(value: i16) -> Self {
-        if value >= 0 { Self::Positive } else { Self::Negative }
-    }
-
-    #[inline]
-    #[must_use]
-    pub fn inverse(self) -> Self {
-        match self {
-            Self::Positive => Self::Negative,
-            Self::Negative => Self::Positive,
-        }
-    }
-}
-
-impl Display for AxisDirection {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Positive => write!(f, "+"),
-            Self::Negative => write!(f, "-"),
-        }
-    }
-}
-
-impl FromStr for AxisDirection {
-    type Err = String;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s {
-            "+" => Ok(Self::Positive),
-            "-" => Ok(Self::Negative),
-            _ => Err(format!("Invalid AxisDirection string: {s}")),
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, EnumDisplay, EnumFromStr)]
-pub enum HatDirection {
-    Up,
-    Left,
-    Right,
-    Down,
-}
-
-impl HatDirection {
-    pub const ALL: [Self; 4] = [Self::Up, Self::Left, Self::Right, Self::Down];
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub enum GamepadAction {
-    Button(u8),
-    Axis(u8, AxisDirection),
-    Hat(u8, HatDirection),
-}
-
-impl Display for GamepadAction {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Button(idx) => write!(f, "Button {idx}"),
-            Self::Axis(idx, direction) => write!(f, "Axis {idx} {direction}"),
-            Self::Hat(idx, direction) => write!(f, "Hat {idx} {direction}"),
-        }
-    }
-}
-
-impl FromStr for GamepadAction {
-    type Err = String;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let err_fn = || format!("Invalid gamepad action string: {s}");
-
-        let mut split = s.split_ascii_whitespace();
-        let Some(input_type) = split.next() else {
-            return Err(err_fn());
-        };
-
-        let Some(idx) = split.next().and_then(|idx| idx.parse().ok()) else {
-            return Err(err_fn());
-        };
-
-        match input_type {
-            "Button" | "button" => Ok(Self::Button(idx)),
-            "Axis" | "axis" => {
-                let Some(direction) = split.next().and_then(|direction| direction.parse().ok())
-                else {
-                    return Err(err_fn());
-                };
-
-                Ok(Self::Axis(idx, direction))
-            }
-            "Hat" | "hat" => {
-                let Some(direction) = split.next().and_then(|direction| direction.parse().ok())
-                else {
-                    return Err(err_fn());
-                };
-
-                Ok(Self::Hat(idx, direction))
-            }
-            _ => Err(err_fn()),
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub enum GenericInput {
-    Keyboard(Keycode),
-    Gamepad { gamepad_idx: u32, action: GamepadAction },
-    Mouse(MouseButton),
-}
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub(crate) struct CanonicalInput(GenericInput);
 
-impl GenericInput {
-    pub(crate) fn canonicalize(self) -> CanonicalInput {
-        match self {
-            Self::Keyboard(keycode) => {
-                CanonicalInput(Self::Keyboard(canonicalize_keycode(keycode)))
-            }
-            _ => CanonicalInput(self),
-        }
-    }
-}
-
 fn canonicalize_keycode(keycode: Keycode) -> Keycode {
     match keycode {
         Keycode::RShift => Keycode::LShift,
@@ -161,6 +29,15 @@ fn canonicalize_keycode(keycode: Keycode) -> Keycode {
 }
 
 impl CanonicalInput {
+    pub(crate) fn canonicalize(input: GenericInput) -> Self {
+        match input {
+            GenericInput::Keyboard(keycode) => {
+                Self(GenericInput::Keyboard(canonicalize_keycode(keycode)))
+            }
+            _ => Self(input),
+        }
+    }
+
     pub(crate) fn reverse_canonicalize(self) -> Option<&'static [GenericInput]> {
         match self.0 {
             GenericInput::Keyboard(Keycode::LShift) => Some(&[
@@ -180,149 +57,9 @@ impl CanonicalInput {
     }
 }
 
-impl Display for GenericInput {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            &Self::Keyboard(keycode) => write!(f, "Key: {}", keycode_to_str(keycode)),
-            Self::Gamepad { gamepad_idx, action } => write!(f, "Gamepad {gamepad_idx}: {action}"),
-            Self::Mouse(mouse_button) => write!(f, "Mouse: {mouse_button:?}"),
-        }
-    }
-}
-
-fn keycode_to_str(keycode: Keycode) -> Cow<'static, str> {
-    match keycode {
-        Keycode::LShift | Keycode::RShift => "Shift".into(),
-        Keycode::LCtrl | Keycode::RCtrl => "Ctrl".into(),
-        Keycode::LAlt | Keycode::RAlt => "Alt".into(),
-        _ => keycode.name().into(),
-    }
-}
-
-fn keycode_from_str(s: &str) -> Option<Keycode> {
-    match s {
-        "Shift" => Some(Keycode::LShift),
-        "Ctrl" => Some(Keycode::LCtrl),
-        "Alt" => Some(Keycode::LAlt),
-        _ => {
-            if s == Keycode::RShift.name().as_str() {
-                Some(Keycode::LShift)
-            } else if s == Keycode::RCtrl.name().as_str() {
-                Some(Keycode::LCtrl)
-            } else if s == Keycode::RAlt.name().as_str() {
-                Some(Keycode::LAlt)
-            } else {
-                Keycode::from_name(s)
-            }
-        }
-    }
-}
-
 pub const MAX_MAPPING_LEN: usize = 3;
 type MappingArrayVec = ArrayVec<CanonicalInput, MAX_MAPPING_LEN>;
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, EnumDisplay, EnumAll)]
-pub enum Hotkey {
-    Exit,
-    ToggleFullscreen,
-    SoftReset,
-    HardReset,
-    PowerOff,
-    Pause,
-    StepFrame,
-    FastForward,
-    Rewind,
-    ToggleOverclocking,
-    OpenDebugger,
-    SaveState,
-    LoadState,
-    NextSaveStateSlot,
-    PrevSaveStateSlot,
-    SaveStateSlot0,
-    LoadStateSlot0,
-    SaveStateSlot1,
-    LoadStateSlot1,
-    SaveStateSlot2,
-    LoadStateSlot2,
-    SaveStateSlot3,
-    LoadStateSlot3,
-    SaveStateSlot4,
-    LoadStateSlot4,
-    SaveStateSlot5,
-    LoadStateSlot5,
-    SaveStateSlot6,
-    LoadStateSlot6,
-    SaveStateSlot7,
-    LoadStateSlot7,
-    SaveStateSlot8,
-    LoadStateSlot8,
-    SaveStateSlot9,
-    LoadStateSlot9,
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub enum CompactHotkey {
-    PowerOff,
-    Exit,
-    ToggleFullscreen,
-    SaveState,
-    LoadState,
-    SaveStateSlot(usize),
-    LoadStateSlot(usize),
-    NextSaveStateSlot,
-    PrevSaveStateSlot,
-    SoftReset,
-    HardReset,
-    Pause,
-    StepFrame,
-    FastForward,
-    Rewind,
-    ToggleOverclocking,
-    OpenDebugger,
-}
-
-impl Hotkey {
-    pub(crate) fn to_compact(self) -> CompactHotkey {
-        match self {
-            Self::PowerOff => CompactHotkey::PowerOff,
-            Self::Exit => CompactHotkey::Exit,
-            Self::ToggleFullscreen => CompactHotkey::ToggleFullscreen,
-            Self::SaveState => CompactHotkey::SaveState,
-            Self::LoadState => CompactHotkey::LoadState,
-            Self::NextSaveStateSlot => CompactHotkey::NextSaveStateSlot,
-            Self::PrevSaveStateSlot => CompactHotkey::PrevSaveStateSlot,
-            Self::SoftReset => CompactHotkey::SoftReset,
-            Self::HardReset => CompactHotkey::HardReset,
-            Self::Pause => CompactHotkey::Pause,
-            Self::StepFrame => CompactHotkey::StepFrame,
-            Self::FastForward => CompactHotkey::FastForward,
-            Self::Rewind => CompactHotkey::Rewind,
-            Self::ToggleOverclocking => CompactHotkey::ToggleOverclocking,
-            Self::OpenDebugger => CompactHotkey::OpenDebugger,
-            Self::SaveStateSlot0 => CompactHotkey::SaveStateSlot(0),
-            Self::SaveStateSlot1 => CompactHotkey::SaveStateSlot(1),
-            Self::SaveStateSlot2 => CompactHotkey::SaveStateSlot(2),
-            Self::SaveStateSlot3 => CompactHotkey::SaveStateSlot(3),
-            Self::SaveStateSlot4 => CompactHotkey::SaveStateSlot(4),
-            Self::SaveStateSlot5 => CompactHotkey::SaveStateSlot(5),
-            Self::SaveStateSlot6 => CompactHotkey::SaveStateSlot(6),
-            Self::SaveStateSlot7 => CompactHotkey::SaveStateSlot(7),
-            Self::SaveStateSlot8 => CompactHotkey::SaveStateSlot(8),
-            Self::SaveStateSlot9 => CompactHotkey::SaveStateSlot(9),
-            Self::LoadStateSlot0 => CompactHotkey::LoadStateSlot(0),
-            Self::LoadStateSlot1 => CompactHotkey::LoadStateSlot(1),
-            Self::LoadStateSlot2 => CompactHotkey::LoadStateSlot(2),
-            Self::LoadStateSlot3 => CompactHotkey::LoadStateSlot(3),
-            Self::LoadStateSlot4 => CompactHotkey::LoadStateSlot(4),
-            Self::LoadStateSlot5 => CompactHotkey::LoadStateSlot(5),
-            Self::LoadStateSlot6 => CompactHotkey::LoadStateSlot(6),
-            Self::LoadStateSlot7 => CompactHotkey::LoadStateSlot(7),
-            Self::LoadStateSlot8 => CompactHotkey::LoadStateSlot(8),
-            Self::LoadStateSlot9 => CompactHotkey::LoadStateSlot(9),
-        }
-    }
-}
-
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum GenericButton<Button> {
     Button(Button, Player),
@@ -437,11 +174,11 @@ where
             self.mappings
                 .entry(generic_button)
                 .or_default()
-                .push(mapping.iter().copied().map(GenericInput::canonicalize).collect());
+                .push(mapping.iter().copied().map(CanonicalInput::canonicalize).collect());
 
             for &mapping_input in mapping {
                 self.inputs_to_buttons
-                    .entry(mapping_input.canonicalize())
+                    .entry(CanonicalInput::canonicalize(mapping_input))
                     .or_default()
                     .push(generic_button);
             }
@@ -457,11 +194,11 @@ where
             self.mappings
                 .entry(generic_button)
                 .or_default()
-                .push(mapping.iter().copied().map(GenericInput::canonicalize).collect());
+                .push(mapping.iter().copied().map(CanonicalInput::canonicalize).collect());
 
             for &mapping_input in mapping {
                 self.inputs_to_buttons
-                    .entry(mapping_input.canonicalize())
+                    .entry(CanonicalInput::canonicalize(mapping_input))
                     .or_default()
                     .push(generic_button);
             }
@@ -477,7 +214,7 @@ where
             return;
         }
 
-        let input = raw_input.canonicalize();
+        let input = CanonicalInput::canonicalize(raw_input);
         if let Some(raw_inputs) = input.reverse_canonicalize() {
             for &other_raw_input in raw_inputs {
                 if other_raw_input == raw_input {
@@ -791,7 +528,7 @@ fn is_hat_direction_pressed(direction: HatDirection, state: HatState) -> bool {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use smsgg_core::{SmsGgButton, SmsGgInputs};
+    use smsgg_config::{SmsGgButton, SmsGgInputs};
     use std::mem;
 
     fn take_hotkey_events<I, B>(state: &mut InputMapperState<I, B>) -> Vec<HotkeyEvent> {
