@@ -20,10 +20,9 @@ pub use snes::{NativeSnesEmulator, create_snes};
 pub use state::{SAVE_STATE_SLOTS, SaveStateMetadata};
 
 use crate::archive::ArchiveError;
-use crate::config::input::ButtonMappingVec;
-use crate::config::{CommonConfig, FullscreenMode, HideMouseCursor, WindowSize};
+use crate::config::CommonConfig;
 use crate::fpstracker::FpsTracker;
-use crate::input::{CompactHotkey, Hotkey, HotkeyEvent, InputMapper, Joysticks};
+use crate::input::{HotkeyEvent, InputMapper, Joysticks};
 use crate::mainloop::audio::SdlAudioOutput;
 use crate::mainloop::debug::{DebugRenderFn, DebuggerWindow};
 use crate::mainloop::rewind::Rewinder;
@@ -55,15 +54,6 @@ use thiserror::Error;
 
 const MODAL_DURATION: Duration = Duration::from_secs(3);
 
-impl FullscreenMode {
-    fn to_sdl_fullscreen(self) -> FullscreenType {
-        match self {
-            Self::Borderless => FullscreenType::Desktop,
-            Self::Exclusive => FullscreenType::True,
-        }
-    }
-}
-
 trait RendererExt {
     fn focus(&mut self);
 
@@ -195,6 +185,7 @@ pub struct NativeEmulator<Emulator: EmulatorTrait> {
     raw_config: Emulator::Config,
     // Config with overclocking maybe forcibly disabled due to hotkey state
     config: Emulator::Config,
+    common_config: CommonConfig,
     renderer: WgpuRenderer<Window>,
     audio_output: SdlAudioOutput,
     input_mapper: InputMapper<Emulator::Inputs, Emulator::Button>,
@@ -347,6 +338,14 @@ pub enum NativeEmulatorError {
     },
     #[error("{0}")]
     Archive(#[from] ArchiveError),
+    #[error("No SMS BIOS provided")]
+    SmsNoBios,
+    #[error("Error opening BIOS file at '{path}': {source}")]
+    SmsBiosRead {
+        path: PathBuf,
+        #[source]
+        source: io::Error,
+    },
     #[error("BIOS is required for Sega CD emulation")]
     SegaCdNoBios,
     #[error("Error opening BIOS file at '{path}': {source}")]
@@ -355,7 +354,6 @@ pub enum NativeEmulatorError {
         #[source]
         source: io::Error,
     },
-
     #[error("{0}")]
     SegaCdDisc(#[from] SegaCdLoadError),
     #[error("{0}")]
@@ -380,8 +378,8 @@ pub enum NativeEmulatorError {
     LoadStateIo(io::Error),
     #[error("Save state begins with invalid prefix")]
     LoadStatePrefixMismatch,
-    #[error("Save state version mismatch; expected {expected}, got {actual}")]
-    LoadStateVersionMismatch { expected: u16, actual: u16 },
+    #[error("Save state version mismatch; expected '{expected}', got '{actual}'")]
+    LoadStateVersionMismatch { expected: String, actual: String },
     #[error("Error in emulation core: {0}")]
     Emulator(#[source] Box<dyn Error + Send + Sync + 'static>),
 }
@@ -406,7 +404,7 @@ where
         initial_inputs: Emulator::Inputs,
         debug_render_fn: fn() -> Box<DebugRenderFn<Emulator>>,
     ) -> NativeEmulatorResult<Self> {
-        let (sdl, video, audio, joystick, event_pump) = init_sdl(&common_config)?;
+        let (sdl, video, audio, joystick, event_pump) = init_sdl2(&common_config)?;
 
         let mut initial_window_size = common_config.window_size.unwrap_or(default_window_size);
         if let Some(scale_factor) = common_config.window_scale_factor {
@@ -446,6 +444,7 @@ where
             emulator,
             raw_config: emulator_config.clone(),
             config: emulator_config,
+            common_config: common_config.clone(),
             renderer,
             audio_output,
             input_mapper,
@@ -499,6 +498,8 @@ where
 
             self.audio_output.adjust_dynamic_resampling_ratio();
             self.emulator.update_audio_output_frequency(self.audio_output.output_frequency());
+
+            self.renderer.set_target_fps(self.emulator.target_fps());
         }
 
         self.hotkey_state.should_step_frame = false;
@@ -806,8 +807,7 @@ fn file_name_no_ext<P: AsRef<Path>>(path: P) -> NativeEmulatorResult<String> {
         .ok_or_else(|| NativeEmulatorError::ParseFileName(path.as_ref().display().to_string()))
 }
 
-// Initialize SDL2
-fn init_sdl(
+fn init_sdl2(
     config: &CommonConfig,
 ) -> NativeEmulatorResult<(Sdl, VideoSubsystem, AudioSubsystem, JoystickSubsystem, EventPump)> {
     let sdl = sdl2::init().map_err(NativeEmulatorError::SdlInit)?;
@@ -816,6 +816,10 @@ fn init_sdl(
     let joystick = sdl.joystick().map_err(NativeEmulatorError::SdlJoystickInit)?;
     let event_pump = sdl.event_pump().map_err(NativeEmulatorError::SdlEventPumpInit)?;
 
+    // Allow gamepad inputs while window does not have focus
+    // https://wiki.libsdl.org/SDL2/SDL_HINT_JOYSTICK_ALLOW_BACKGROUND_EVENTS
+    sdl2::hint::set("SDL_JOYSTICK_ALLOW_BACKGROUND_EVENTS", "1");
+
     sdl.mouse().show_cursor(!config.hide_mouse_cursor.should_hide(config.launch_in_fullscreen));
 
     Ok((sdl, video, audio, joystick, event_pump))
@@ -864,3 +868,6 @@ macro_rules! bincode_config {
 }
 
 use bincode_config;
+use jgenesis_native_config::common::{FullscreenMode, HideMouseCursor, WindowSize};
+use jgenesis_native_config::input::mappings::ButtonMappingVec;
+use jgenesis_native_config::input::{CompactHotkey, Hotkey};
