@@ -2,10 +2,10 @@ use crate::config::GameBoyConfig;
 use crate::config::RomReadResult;
 use crate::mainloop::save::{DeterminedPaths, FsSaveWriter};
 use crate::mainloop::{debug, file_name_no_ext, save};
-use crate::{AudioError, NativeEmulator, NativeEmulatorResult, config, extensions};
+use crate::{AudioError, NativeEmulator, NativeEmulatorResult, extensions};
+use gb_config::GameBoyInputs;
 use gb_core::api::GameBoyEmulator;
-use gb_core::inputs::GameBoyInputs;
-use jgenesis_common::frontend::EmulatorTrait;
+use jgenesis_native_config::common::WindowSize;
 use std::path::Path;
 
 pub type NativeGameBoyEmulator = NativeEmulator<GameBoyEmulator>;
@@ -21,9 +21,6 @@ impl NativeGameBoyEmulator {
 
         self.update_emulator_config(&config.emulator_config);
 
-        // Config change could have changed target framerate (60 Hz hack)
-        self.renderer.set_target_fps(self.emulator.target_fps());
-
         self.input_mapper.update_mappings(
             config.common.axis_deadzone,
             &config.inputs.to_mapping_vec(),
@@ -60,12 +57,14 @@ pub fn create_gb(config: Box<GameBoyConfig>) -> NativeEmulatorResult<NativeGameB
     let rom_title = file_name_no_ext(&config.common.rom_file_path)?;
     let window_title = format!("gb - {rom_title}");
 
+    let default_window_size = WindowSize::new_gb(config.common.initial_window_size);
+
     NativeGameBoyEmulator::new(
         emulator,
         emulator_config,
         config.common,
         extension,
-        config::DEFAULT_GB_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
