@@ -2,13 +2,14 @@ use crate::config::RomReadResult;
 use crate::config::{GenesisConfig, Sega32XConfig, SegaCdConfig};
 use crate::mainloop::save::{DeterminedPaths, FsSaveWriter};
 use crate::mainloop::{NativeEmulatorError, debug, save};
-use crate::{AudioError, NativeEmulator, NativeEmulatorResult, config, extensions};
+use crate::{AudioError, NativeEmulator, NativeEmulatorResult, extensions};
 use genesis_core::{GenesisEmulator, GenesisInputs};
+use jgenesis_native_config::common::WindowSize;
 use s32x_core::api::Sega32XEmulator;
 use segacd_core::CdRomFileFormat;
 use segacd_core::api::{SegaCdEmulator, SegaCdLoadResult};
 use std::fs;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 
 pub type NativeGenesisEmulator = NativeEmulator<GenesisEmulator>;
 
@@ -81,6 +82,7 @@ impl NativeSegaCdEmulator {
             CdRomFileFormat::CueBin
         });
 
+        self.rom_path = rom_path.as_ref().to_path_buf();
         self.emulator.change_disc(rom_path, rom_format)?;
 
         let title = format!("sega cd - {}", self.emulator.disc_title());
@@ -93,6 +95,10 @@ impl NativeSegaCdEmulator {
                 .expect("Disc title should have non-printable characters already removed");
         }
 
+        if let Err(err) = self.update_save_paths(&self.common_config.clone()) {
+            log::error!("Error updating save paths on disc change: {err}");
+        }
+
         Ok(())
     }
 }
@@ -150,12 +156,18 @@ pub fn create_genesis(config: Box<GenesisConfig>) -> NativeEmulatorResult<Native
     });
     let window_title = format!("genesis - {cartridge_title}");
 
+    let default_window_size = WindowSize::new_genesis(
+        config.common.initial_window_size,
+        emulator_config.aspect_ratio,
+        emulator.timing_mode(),
+    );
+
     NativeGenesisEmulator::new(
         emulator,
         emulator_config,
         config.common,
         extension,
-        config::DEFAULT_GENESIS_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
@@ -176,25 +188,47 @@ pub fn create_sega_cd(config: Box<SegaCdConfig>) -> NativeEmulatorResult<NativeS
 
     log::info!("Running with config: {config}");
 
-    let rom_path = Path::new(&config.genesis.common.rom_file_path);
-    let rom_format = CdRomFileFormat::from_file_path(rom_path).unwrap_or_else(|| {
-        log::warn!(
-            "Unrecognized CD-ROM file extension, behaving as if this is a CUE file: {}",
-            rom_path.display()
-        );
-        CdRomFileFormat::CueBin
-    });
+    let bios_file_path = config.bios_file_path.as_ref().ok_or(NativeEmulatorError::SegaCdNoBios)?;
 
-    let DeterminedPaths { save_path, save_state_path } = save::determine_save_paths(
-        &config.genesis.common.save_path,
-        &config.genesis.common.state_path,
-        rom_path,
-        SCD_SAVE_EXTENSION,
-    )?;
+    let rom_path: &Path;
+    let rom_format: CdRomFileFormat;
+    let save_path: PathBuf;
+    let save_state_path: PathBuf;
+
+    if !config.run_without_disc {
+        rom_path = Path::new(&config.genesis.common.rom_file_path);
+        rom_format = CdRomFileFormat::from_file_path(rom_path).unwrap_or_else(|| {
+            log::warn!(
+                "Unrecognized CD-ROM file extension, behaving as if this is a CUE file: {}",
+                rom_path.display()
+            );
+            CdRomFileFormat::CueBin
+        });
+
+        let determined_paths = save::determine_save_paths(
+            &config.genesis.common.save_path,
+            &config.genesis.common.state_path,
+            rom_path,
+            SCD_SAVE_EXTENSION,
+        )?;
+        save_path = determined_paths.save_path;
+        save_state_path = determined_paths.save_state_path;
+    } else {
+        rom_path = Path::new("");
+        rom_format = CdRomFileFormat::CueBin;
+
+        let determined_paths = save::determine_save_paths(
+            &config.genesis.common.save_path,
+            &config.genesis.common.state_path,
+            bios_file_path,
+            SCD_SAVE_EXTENSION,
+        )?;
+        save_path = determined_paths.save_path;
+        save_state_path = determined_paths.save_state_path;
+    }
 
     let mut save_writer = FsSaveWriter::new(save_path);
 
-    let bios_file_path = config.bios_file_path.as_ref().ok_or(NativeEmulatorError::SegaCdNoBios)?;
     let bios = fs::read(bios_file_path).map_err(|source| NativeEmulatorError::SegaCdBiosRead {
         path: bios_file_path.clone(),
         source,
@@ -212,12 +246,18 @@ pub fn create_sega_cd(config: Box<SegaCdConfig>) -> NativeEmulatorResult<NativeS
 
     let window_title = format!("sega cd - {}", emulator.disc_title());
 
+    let default_window_size = WindowSize::new_genesis(
+        config.genesis.common.initial_window_size,
+        emulator_config.genesis.aspect_ratio,
+        emulator.timing_mode(),
+    );
+
     NativeSegaCdEmulator::new(
         emulator,
         emulator_config,
         config.genesis.common,
         SCD_SAVE_EXTENSION.into(),
-        config::DEFAULT_GENESIS_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
@@ -255,12 +295,18 @@ pub fn create_32x(config: Box<Sega32XConfig>) -> NativeEmulatorResult<Native32XE
     let cartridge_title = emulator.cartridge_title();
     let window_title = format!("32x - {cartridge_title}");
 
+    let default_window_size = WindowSize::new_32x(
+        config.genesis.common.initial_window_size,
+        emulator_config.genesis.aspect_ratio,
+        emulator.timing_mode(),
+    );
+
     Native32XEmulator::new(
         emulator,
         emulator_config,
         config.genesis.common,
         extension,
-        config::DEFAULT_GENESIS_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
