@@ -2,14 +2,15 @@ use crate::config::NesConfig;
 
 use crate::mainloop::save::{DeterminedPaths, FsSaveWriter};
 use crate::mainloop::{debug, file_name_no_ext, save};
-use crate::{AudioError, NativeEmulator, NativeEmulatorResult, config, extensions};
-use jgenesis_common::frontend::EmulatorTrait;
+use crate::{AudioError, NativeEmulator, NativeEmulatorResult, extensions};
 
 use nes_core::api::NesEmulator;
-use nes_core::input::{NesInputDevice, NesInputs, NesJoypadState, ZapperState};
+use nes_core::input::{NesInputDevice, NesInputs, ZapperState};
 
 use crate::config::RomReadResult;
-use crate::config::input::NesControllerType;
+use jgenesis_native_config::common::WindowSize;
+use jgenesis_native_config::input::mappings::NesControllerType;
+use nes_config::NesJoypadState;
 use std::path::Path;
 
 trait NesControllerTypeExt {
@@ -38,9 +39,6 @@ impl NativeNesEmulator {
 
         self.update_emulator_config(&config.emulator_config);
 
-        // Config change could have changed target framerate (50/60 Hz hack)
-        self.renderer.set_target_fps(self.emulator.target_fps());
-
         self.input_mapper.update_mappings(
             config.common.axis_deadzone,
             &config.inputs.to_mapping_vec(),
@@ -81,12 +79,19 @@ pub fn create_nes(config: Box<NesConfig>) -> NativeEmulatorResult<NativeNesEmula
     let initial_inputs =
         NesInputs { p1: NesJoypadState::default(), p2: config.inputs.p2_type.to_input_device() };
 
+    let default_window_size = WindowSize::new_nes(
+        config.common.initial_window_size,
+        emulator_config.aspect_ratio,
+        emulator.timing_mode(),
+        emulator_config.ntsc_crop_vertical_overscan,
+    );
+
     NativeNesEmulator::new(
         emulator,
         emulator_config,
         config.common,
         extension,
-        config::DEFAULT_GENESIS_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
