@@ -2,13 +2,14 @@ use crate::config::SnesConfig;
 
 use crate::mainloop::save::{DeterminedPaths, FsSaveWriter};
 use crate::mainloop::{debug, save};
-use crate::{AudioError, NativeEmulator, NativeEmulatorResult, config, extensions};
-use jgenesis_common::frontend::EmulatorTrait;
+use crate::{AudioError, NativeEmulator, NativeEmulatorResult, extensions};
 
 use crate::config::RomReadResult;
-use crate::config::input::SnesControllerType;
+use jgenesis_native_config::common::WindowSize;
+use jgenesis_native_config::input::mappings::SnesControllerType;
+use snes_config::SnesJoypadState;
 use snes_core::api::SnesEmulator;
-use snes_core::input::{SnesInputDevice, SnesInputs, SnesJoypadState, SuperScopeState};
+use snes_core::input::{SnesInputDevice, SnesInputs, SuperScopeState};
 use std::path::Path;
 
 trait SnesControllerTypeExt {
@@ -37,9 +38,6 @@ impl NativeSnesEmulator {
 
         self.update_emulator_config(&config.emulator_config);
 
-        // Config change could have changed target framerate (50/60 Hz hack)
-        self.renderer.set_target_fps(self.emulator.target_fps());
-
         self.input_mapper.update_mappings(
             config.common.axis_deadzone,
             &config.inputs.to_mapping_vec(),
@@ -82,12 +80,15 @@ pub fn create_snes(config: Box<SnesConfig>) -> NativeEmulatorResult<NativeSnesEm
     let initial_inputs =
         SnesInputs { p1: SnesJoypadState::default(), p2: config.inputs.p2_type.to_input_device() };
 
+    let default_window_size =
+        WindowSize::new_snes(config.common.initial_window_size, emulator_config.aspect_ratio);
+
     NativeSnesEmulator::new(
         emulator,
         emulator_config,
         config.common,
         extension,
-        config::DEFAULT_GENESIS_WINDOW_SIZE,
+        default_window_size,
         &window_title,
         save_writer,
         save_state_path,
