@@ -18,7 +18,8 @@ use jgenesis_common::frontend::{
 use jgenesis_renderer::renderer::{WgpuRenderer, WindowSize};
 use rfd::AsyncFileDialog;
 use segacd_core::api::SegaCdEmulator;
-use smsgg_core::{SmsGgEmulator, SmsGgHardware, SmsGgInputs};
+use smsgg_config::SmsGgInputs;
+use smsgg_core::{SmsGgEmulator, SmsGgHardware};
 use snes_core::api::{CoprocessorRoms, SnesEmulator};
 use snes_core::input::SnesInputs;
 use std::collections::HashMap;
@@ -142,7 +143,7 @@ impl SaveWriter for LocalStorageSaveWriter {
         Ok(())
     }
 
-    fn load_serialized<D: Decode>(&mut self, extension: &str) -> Result<D, Self::Err> {
+    fn load_serialized<D: Decode<()>>(&mut self, extension: &str) -> Result<D, Self::Err> {
         let file_name = self.get_file_name(extension);
         let bytes = read_save_file(&file_name)?;
         let (value, _) = bincode::decode_from_slice(&bytes, bincode_config!())
@@ -843,7 +844,8 @@ fn open_emulator(
                 _ => unreachable!("nested match expressions"),
             };
             let emulator = SmsGgEmulator::create(
-                rom,
+                Some(rom),
+                None,
                 hardware,
                 config_ref.borrow().smsgg.to_emulator_config(),
                 save_writer,
